// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.collection;
/** tests non-destructive list implementation with String */

public class TestSLListOfString extends junit.framework.TestCase {

    String str[]=new String[]{"Dies","ist","ein","Test"};

    public TestSLListOfString(String name) {
	super(name);
    }

    ListOfString a;   // "A" "B" "C"
    ListOfString a1;  // "A" "B" "C"
    ListOfString b;   // "A" "B"
    ListOfString c;   // "A" "B" "C" "D"
    ListOfString d;   // "A" "B" "A"
    ListOfString e;   // "A" "B" null
    ListOfString e1;   // "A" "B" null


    public void setUp() {
	a = SLListOfString.EMPTY_LIST
	    .prepend("C").prepend("B").prepend("A");
	a1 = SLListOfString.EMPTY_LIST
	    .prepend("C").prepend("B").prepend("A");
	b = SLListOfString.EMPTY_LIST
	    .prepend("B").prepend("A");
	c = SLListOfString.EMPTY_LIST
	    .prepend("D").prepend("C").prepend("B").prepend("A");
	d = SLListOfString.EMPTY_LIST
	    .prepend("A").prepend("B").prepend("A");
	e = SLListOfString.EMPTY_LIST
	    .prepend((String)null).prepend("B").prepend("A");
	e1 = SLListOfString.EMPTY_LIST
	    .prepend((String)null).prepend("B").prepend("A");
    }

    // tests prepend and implicitly iterator, size 
    public void testPrepend() {
	ListOfString[] newList=new ListOfString[str.length+1];
	newList[0]=SLListOfString.EMPTY_LIST;
	
	for (int i=1;i<str.length+1;i++) {
	    newList[i]=newList[i-1].prepend(str[i-1]);	    
	}
	// Test elements in list
	for (int i=0;i<str.length+1;i++) {
	    IteratorOfString it=newList[i].iterator();
	    int size=newList[i].size();
	    if (i>0) { // list should have elements 
		assert(it.hasNext());
		assert(size==i);
	    } else { // list is empty
		assert(!it.hasNext());
		assert(size==0);
	    }	    
	    int nr=0;
	    while (it.hasNext()) {
		assertSame(it.next(),str[size-1-nr]);	    
		nr++;
	    }
	    // list has right length
	    assert(nr==size);
	}
	// prepend two lists
	ListOfString prepList=newList[1].prepend(newList[2]);
	assert(prepList.size()==3);
	// right order
	assertEquals(str[1],prepList.head());
	assertEquals(str[0],prepList.tail().head());
	assertEquals(str[0],prepList.tail().tail().head());
    }

    // tests append and implicitly iterator, size 
    public void testAppend() {
	ListOfString[] newList=new ListOfString[str.length+1];
	newList[0]=SLListOfString.EMPTY_LIST;
	
	for (int i=1;i<str.length+1;i++) {
	    newList[i]=newList[i-1].append(str[i-1]);	    
	}
	// Test elements in list
	for (int i=0;i<str.length+1;i++) {	    
	    IteratorOfString it=newList[i].iterator();
	    int size=newList[i].size();
	    if (i>0) { // list should have elements 
		assert(it.hasNext());
		assert(size==i);
	    } else { // list is empty
		assert(!it.hasNext());
		assert(size==0);
	    }	    
	    int nr=0;
	    while (it.hasNext()) {
		assertSame(it.next(),str[nr]);	    
		nr++;
	    }
	    // list has right length
	    assert(nr==size);
	}

	// append two lists
	ListOfString appList=newList[2].append(newList[1]);
	assert(appList.size()==3);
	// right order
	assertEquals(str[0],appList.head());
	assertEquals(str[1],appList.tail().head());
	assertEquals(str[0],appList.tail().tail().head());
    }

    // tests tail,head
    public void testHeadTail() {
	ListOfString[] newList=new ListOfString[str.length+1];
	newList[0]=SLListOfString.EMPTY_LIST;	

	for (int i=1;i<str.length+1;i++) {
	    newList[i]=newList[i-1].prepend(str[i-1]);	    
	}	
	// test cascading tail
	for (int i=0;i<str.length;i++) {
	    assertSame(newList[i+1].tail(),newList[i]);
	    assertSame(newList[i+1].head(),str[i]);
	}	
    }

   // tests contains
    public void testContains() {
	ListOfString newList=SLListOfString.EMPTY_LIST;	

	for (int i=1;i<str.length+1;i++) {
	    newList=newList.append(str[i-1]);	    
	}	
	// test cascading tail
	for (int i=0;i<str.length;i++) {
	    assert(newList.contains(str[i]));
	}	
    }


  // tests removeAll
    public void testRemoveAll() {
	ListOfString newList=SLListOfString.EMPTY_LIST;	

	newList=newList.append(str[0]);
	for (int i=1;i<str.length+1;i++) {
	    newList=newList.append(str[i-1]);	    
	}	
	newList=newList.append(str[0]);
	newList=newList.removeAll(str[0]);
	assert("str[0] should have been removed",!newList.contains(str[0]));

    }   

    public void testEquals() {
	assert("a==a1",a.equals(a1));
	assert("a!=b",! a.equals(b));
	assert("a!=c",! a.equals(c));
	assert("a!=d",! a.equals(d));
	assert("a!=e",! a.equals(e));
	assert("e!=a",! e.equals(a));
	assert("e==e1", e.equals(e1));
    }
    

    public void testToString() {
	ListOfString newList=SLListOfString.EMPTY_LIST;	
	for (int i=0;i<str.length;i++) {
	    newList=newList.append(str[i]);	    
	}	
	assertEquals("[Dies,ist,ein,Test]",newList.toString());
    }


}
