// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.gui;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;

import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.logic.Namespace;
import de.uka.ilkd.key.rule.Heuristic;
import de.uka.ilkd.key.rule.ListOfHeuristic;
import de.uka.ilkd.key.rule.SLListOfHeuristic;
import de.uka.ilkd.key.rule.IteratorOfHeuristic;
import de.uka.ilkd.key.logic.IteratorOfNamed;

public class HeuristicSelector extends JDialog {

    private HeuristicSettings settingsFromConfigurationFile;
    
    private HeuristicListModel notSelectedListModel;
    private HeuristicListModel activeHeuristicsListModel;
    private JList notSelectedHeuristics;
    private JList activeHeuristics;
    private LinkedList selected;
    private LinkedList notSelected;
    private JButton toRight;
    private JButton toLeft;
    private JButton allToRight;
    private JButton allToLeft;
    private JTextField maxHeuristicStepsInputField;

    private ListOfHeuristic originalSelectedList;
    private KeYMediator mediator = null;

    /** creates a new HeuristicSelector */
    public HeuristicSelector(KeYMediator mediator, 
			     Namespace heur_nsp, 
			     HeuristicSettings settings) {  
	super(new JFrame(), "Activate Heuristics", true);

	settingsFromConfigurationFile = settings;
	setMediator(mediator);

	// reuse the settings from the configuration file if present ...
	Debug.out("RESETTING the heuristics settings from the heuristics-configuration-file !");
	maxHeuristicStepsInputField = 
	    new NumberInputField(settingsFromConfigurationFile.getMaxSteps(), 4);	    
	createSelectedList(settingsFromConfigurationFile.
			   getActiveHeuristics(heur_nsp));
	createNotSelectedList(heur_nsp);
	
	layoutHeuristicSelector(); 
	
	pack();
	setLocation(70,70);
	setVisible(true);

    }

    /** creates a linked list with same content like sel
     * @param sel ListOfHeuristic to be converted
     */
    private void createSelectedList(ListOfHeuristic sel) {
        selected = new LinkedList();
	IteratorOfHeuristic it = sel.iterator();
	while (it.hasNext()) {
	    Heuristic h = it.next();
	    selected.addLast(h);	    
	}
    }
    /** creates a linked list from the heuristics in the given
     * namespace without the heuristics that are found in the list of
     * selected(active) heuristics
     * @param heur_nsp the Namespace with all known heuristics
     */
    private void createNotSelectedList(Namespace heur_nsp) {
        notSelected = new LinkedList();
	Namespace h_nsp = heur_nsp;
	while (h_nsp != null) {
	    IteratorOfNamed it = h_nsp.elements().iterator();
	    while (it.hasNext()) {
		Heuristic next = (Heuristic)it.next();
		if (!selected.contains(next)) {
		    notSelected.addLast(next);
		}
	    }
	    h_nsp = h_nsp.parent();
	}
    }

    private void updateButtons() {
	toRight.setEnabled(!notSelectedHeuristics.isSelectionEmpty());	
	toLeft.setEnabled(!activeHeuristics.isSelectionEmpty());	  
	allToRight.setEnabled(notSelectedListModel.getSize()>0);	
	allToLeft.setEnabled(activeHeuristicsListModel.getSize()>0);	  
    }


    
    private void makeActive(Object[] notSelected) {
	if (notSelected==null || notSelected.length==0) {
	    return;
	}
	activeHeuristicsListModel.add(notSelected);
	activeHeuristics.setSelectedValue(notSelected[0],true);
	notSelectedListModel.remove(notSelected);	
	updateButtons();
    }

    private void makeInactive(Object[] selected) {
	if (selected==null || selected.length==0) {
	    return;
	}
	notSelectedListModel.add(selected);
	notSelectedHeuristics.setSelectedValue(selected[0],true);

	activeHeuristicsListModel.remove(selected);
	updateButtons();
    }

    /** lays out the selector */
    protected void layoutHeuristicSelector() { 
	getContentPane().setLayout(new BorderLayout());

	notSelectedListModel = new HeuristicListModel(notSelected);
	notSelectedHeuristics = new JList(notSelectedListModel);
	notSelectedHeuristics.setSelectionMode(ListSelectionModel.
					       MULTIPLE_INTERVAL_SELECTION);
	notSelectedHeuristics.setBackground(Color.white);

	activeHeuristicsListModel = new HeuristicListModel(selected);
	activeHeuristics = new JList(activeHeuristicsListModel); 
	activeHeuristics.setSelectionMode(ListSelectionModel.
					  MULTIPLE_INTERVAL_SELECTION);
	activeHeuristics.setBackground(Color.white);	

	notSelectedHeuristics.addMouseListener(new MouseAdapter() {
	
		public void mouseClicked(MouseEvent e) {
		    updateButtons();
		    if (e.getClickCount() == 2) {
			makeActive(notSelectedHeuristics.getSelectedValues());	      
		    }
		}
	    });

	activeHeuristics.addMouseListener(new MouseAdapter() {

		public void mouseClicked(MouseEvent e) {
		    updateButtons();
		    if (e.getClickCount() == 2) {
			makeInactive(activeHeuristics.getSelectedValues());
		    }
		}
	    });
	
	JScrollPane notSelScrollPane = new
	    JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	notSelScrollPane.getViewport().setView(notSelectedHeuristics);
	notSelScrollPane.setBorder(new TitledBorder("Possible Heuristics"));

	JScrollPane activeScrollPane = new 	    
	    JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

	activeScrollPane.getViewport().setView(activeHeuristics);
	activeScrollPane.setBorder(new TitledBorder("Active Heuristics"));


 	JPanel maxInput=new JPanel();
 	maxInput.setLayout(new BoxLayout(maxInput, BoxLayout.X_AXIS));
 	maxHeuristicStepsInputField = new NumberInputField(mediator().getMaxHeuristicSteps(), 4);
	maxHeuristicStepsInputField.setMaximumSize(new Dimension(40,30));
 	maxInput.add(new JLabel("Maximum heuristics applications: "));
	maxInput.add(maxHeuristicStepsInputField);

 	JButton okButton = new JButton("OK");
	okButton.setMnemonic(KeyEvent.VK_ENTER);

 	okButton.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e) {
		    setVisible(false);
		    int maxSteps = Integer.parseInt(maxHeuristicStepsInputField.getText());
		    ListOfHeuristic activeHeuristics = activeHeuristics();
		    settingsFromConfigurationFile.setMaxSteps(maxSteps);
		    settingsFromConfigurationFile.setHeuristics(activeHeuristics);		    
		    mediator().setMaxHeuristicSteps(maxSteps);
		    mediator().setActiveHeuristics(activeHeuristics);
 		}
	    });
 	JButton cancelButton = new JButton("Cancel");
	cancelButton.setMnemonic(KeyEvent.VK_ESCAPE);
 	cancelButton.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e) {
		    setVisible(false);
		    dispose();
 		}
 	    });

 	JPanel buttonPanel = new JPanel();
 	buttonPanel.add(okButton);
 	buttonPanel.add(cancelButton);
	
	JPanel labelPanel = new JPanel();
//	labelPanel.setLayout(new GridLayout(5,1));
	toRight=new JButton(">");
	toRight.setToolTipText("Activate heuristic");
	toRight.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e) {
		    makeActive(notSelectedHeuristics.getSelectedValues());
 		}
 	    });
	toLeft=new JButton("<");
	toLeft.setToolTipText("Deactivate heuristic");
	toLeft.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e) {
		    makeInactive(activeHeuristics.getSelectedValues());
 		}
 	    });



	allToRight=new JButton(">>");
	allToRight.setToolTipText("Activate all");
	allToRight.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e) {
		    makeActive(notSelectedListModel.getAllElements());
 		}
 	    });
	allToLeft=new JButton("<<");
	allToLeft.setToolTipText("Deactivate all");
	allToLeft.addActionListener(new ActionListener() {
 		public void actionPerformed(ActionEvent e) {
		    makeInactive(activeHeuristicsListModel.getAllElements());
 		}
 	    });




	labelPanel.add(allToRight);
	labelPanel.add(new JLabel(""));
	labelPanel.add(toRight);
	labelPanel.add(new JLabel(""));
	labelPanel.add(toLeft);
	labelPanel.add(new JLabel(""));
	labelPanel.add(allToLeft);
	labelPanel.setSize(30,100);

	JPanel listPanel=new JPanel();

	listPanel.setLayout(new BoxLayout(listPanel,BoxLayout.X_AXIS));
	notSelScrollPane.setPreferredSize(new Dimension(250, 150));
        notSelScrollPane.setMinimumSize(new Dimension(250, 150));

	labelPanel.setPreferredSize(new Dimension(80, 150));
        labelPanel.setMinimumSize(new Dimension(80, 150));
	labelPanel.setMaximumSize(new Dimension(80, 150));
	activeScrollPane.setPreferredSize(new Dimension(250, 150));
        activeScrollPane.setMinimumSize(new Dimension(250, 150));


	listPanel.add(notSelScrollPane);
	listPanel.add(labelPanel);
	listPanel.add(activeScrollPane);

	getContentPane().setLayout(new BoxLayout(getContentPane(), 
						 BoxLayout.Y_AXIS));
	getContentPane().add(listPanel);
	getContentPane().add(maxInput);
	getContentPane().add(buttonPanel);

	updateButtons();

    }
    
    /** sets the mediator to correspond with other gui elements
     * @param mediator the KeYMediator
     */
    public void setMediator(KeYMediator mediator) {	
	this.mediator = mediator;
    }

    /** returns the KeYMediator used to communicate with other gui
     * elements
     * @return mediator the KeYMediator
     */
    public KeYMediator mediator() {
	return mediator;
    }
    
    /** converts the list with all active heuristics to a
     * ListOfHeuristic Representation 
     * @return ListOfHeuristic with the active heuristics
     */
    private ListOfHeuristic activeHeuristics() {
	ListOfHeuristic result = SLListOfHeuristic.EMPTY_LIST;
	for (int i = selected.size()-1; i >= 0 ; i--) {
	    result = result.prepend((Heuristic)selected.get(i)); 
	}
	return result;
     }

    // INNER CLASS
    
    /** model of the heuristic lists used for JList*/
    static class HeuristicListModel extends AbstractListModel {
    
	private LinkedList heur_list;
	
	HeuristicListModel(LinkedList heur_list) {
	    this.heur_list = heur_list;
	}
	
	public int getSize() {
	    return heur_list.size();
	}
	public void remove(Object obj) {
	    int index = heur_list.indexOf(obj);
	    heur_list.remove(obj);

	    fireIntervalRemoved(this,
				index,
				index+1);
	}

	public void add(Object obj) {
	    heur_list.addLast(obj);
	    fireIntervalAdded(this,
			      heur_list.size()-1,
			      heur_list.size());

	}

	public void add(Object[] obj) {
	    for (int i=0; i<obj.length; i++) {
		add(obj[i]);
	    }
	}


	public void remove(Object[] obj) {
	    for (int i=0; i<obj.length; i++) {
		remove(obj[i]);
	    }
	}


	public Object getElementAt(int i) {
	    return heur_list.get(i);
	}
        

	public Object[] getAllElements() {
	    return heur_list.toArray();
	}
        

    }

    // INNER CLASS TO READ ONLY NUMBERS FOR MAX APPs
    static class NumberDocument extends PlainDocument {

	public void insertString(int offs, String str, AttributeSet a) 
	    throws BadLocationException {
	    if (str == null) {
		return;
	    }
	    char[] upper = str.toCharArray();
	    for (int i = 0; i < upper.length; i++) {
		if (upper[i] < '0' || upper[i] > '9') {
		    return;
		} 
	    }
	    super.insertString(offs, new String(upper), a);
	}
	
    }

    static class NumberInputField extends JTextField {
	public NumberInputField(int number, int cols) {
	    super(cols);
	    setText(""+number);
	}

	protected Document createDefaultModel() {
	    return new NumberDocument();
	}
    }



}    
