// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.java;


import java.util.HashMap;
import de.uka.ilkd.key.util.Debug;



public class KeYRecoderMapping{



    /** maps a recoder programelement (or something similar, e.g. Type)
    * to the KeY-equivalent
    */
    private HashMap map;

    /** maps a KeY programelement to the Recoder-equivalent */
    private HashMap revMap;


    public KeYRecoderMapping() {
	this.map = new HashMap();
	this.revMap = new HashMap();
    }


    /**
    * creates a KeYRecoderMapping object
    * @param map a HashMap mapping ProgramElements in Recoder to
    * ProgramElements in KeY
    * @param revMap the reverse map (KeY->Recoder)
    */
    public KeYRecoderMapping(HashMap map, HashMap revMap){
        this.map      = map;
        this.revMap   = revMap;
    }

    /**
    * returns a matching ProgramElement (KeY) to a given
    * ProgramElement (Recoder)
    * @param pe a recoder.java.ProgramElement
    */
    public ProgramElement toKeY(recoder.java.ProgramElement pe) {
        return (ProgramElement)map.get(pe);
    }

    /**
    * returns a matching ModelElement (KeY) to a given recoder.ModelElement
    * @param pe a recoder.ModelElement
    */
    public ModelElement toKeY(recoder.ModelElement pe) {
        return (ModelElement)map.get(pe);
    }

    /**
    * returns the Recoder-equivalent to a given ProgramElement (KeY).
    * If there's no Recoder-equivalent to the ProgramElement pe
    * a debug message "Program Element <pe> not known" is printed.
    * @param pe a JavaProgramElement
    */
    public recoder.java.ProgramElement toRecoder(ProgramElement pe) {
        Object res=revMap.get(pe);
        Debug.assert(res!=null, "Program Element "+pe+" not known");
        return (recoder.java.ProgramElement)res;
    }


    /**
    * returns the Recoder-equivalent to a given ModelElement (KeY).
    * If there's no Recoder-equivalent to the ModelElement pe a
    * debug message "Model Element <pe> not known" is printed.
    * @param pe a ModelElement
    */
    public recoder.ModelElement toRecoder(ModelElement pe) {
        Object res=revMap.get(pe);
        Debug.assert(res!=null, "Model Element "+pe+" not known");
        return (recoder.ModelElement)res;
    }

    public void put(Object rec, Object key) {
	map.put(rec, key);
	revMap.put(key, rec);
    }

    public boolean mapped(Object rec) {
	return map.containsKey(rec);
    }

    public java.util.Set elemsKeY() {
	return revMap.keySet();
    }

    public KeYRecoderMapping copy() {
	return new KeYRecoderMapping((HashMap)map.clone(), 
				     (HashMap)revMap.clone());
    }

}
