// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.java;


import java.io.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.declaration.modifier.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.recoderext.SchemaJavaProgramFactory;
import de.uka.ilkd.key.java.recoderext.SchemaCrossReferenceServiceConfiguration;
import de.uka.ilkd.key.java.recoderext.TypeSVWrapper;
import de.uka.ilkd.key.java.recoderext.ProgramVariableSVWrapper;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.util.ExtList;
import java.util.Iterator;
import de.uka.ilkd.key.rule.metaconstruct.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.rule.ContextTerm;

public class SchemaRecoder2KeY extends Recoder2KeY implements SchemaJavaReader{

    /** the namespace containing the program schema variables allowed here*/
    protected Namespace svns;
    public static KeYJavaType typeSVType = new KeYJavaType
	(PrimitiveType.PROGRAM_SV, ProgramSVSort.TYPE);

    public SchemaRecoder2KeY(KeYRecoderMapping krm, TypeConverter tc) {
	super(new SchemaCrossReferenceServiceConfiguration(),
	      null, null, krm, tc);
    }

    public SchemaRecoder2KeY(recoder.java.JavaProgramElement progElem,
			     String classPath, KeYRecoderMapping krm, 
			     TypeConverter tc) {
	super((de.uka.ilkd.key.java.recoderext
			  .KeYCrossReferenceServiceConfiguration)
	      new SchemaCrossReferenceServiceConfiguration(), 
	      progElem, classPath, krm, tc);
    }

    public SchemaRecoder2KeY(recoder.java.JavaProgramElement progElem,
			     KeYRecoderMapping krm, TypeConverter tc) {
	this(progElem, null, krm, tc);
    }

    public SchemaRecoder2KeY(String classPath, KeYRecoderMapping krm, 
			     TypeConverter tc) {
	this(null, classPath, krm, tc);
    }

    public void setSVNamespace(Namespace svns) {
	this.svns=svns;
    }

    /** creates an empty RECODER compilation unit 
     * @return the recoder.java.CompilationUnit 
     */
    public Context createEmptyContext() {
	return new Context(new recoder.java.CompilationUnit(), 
			   new recoder.java.declaration.ClassDeclaration());
    }

    protected ProgramMetaConstruct 
	convert(de.uka.ilkd.key.java.recoderext.RKeYMetaConstruct mc) {
	ExtList list=new ExtList();
	try {
	    if (mc.getName().equals("#unwind-loop")) {
		return new UnwindLoop
		    ((LoopStatement)callConvertMethod(list, mc.getChild())
		     .get(LoopStatement.class));	    
	    }
	    if (mc.getName().equals("#try-throw-to-if")) {	
		return new TryThrowToIf
		    ((Try)callConvertMethod(list, mc.getChild() ).get(Try.class));
	    } 	
	    if (mc.getName().equals("#empty-try")) {	
		return new EmptyTryToIf
		    ((Try)callConvertMethod(list, mc.getChild()).get(Try.class));
	    } 	
	    if (mc.getName().equals("#precede-finally")) {	
		return new TryReturnFinally
		    ((Try)callConvertMethod(list, mc.getChild()).get(Try.class));
	    } 	
	    if (mc.getName().equals("#do-break")) {	
		return new DoBreak
		    ((LabeledStatement)callConvertMethod
		     (list, mc.getChild()).get(LabeledStatement.class));
	    } 	
	    if (mc.getName().equals("#expand-method-body")) {	
		return new ExpandMethodBody
		    ((SchemaVariable)callConvertMethod
		     (list, mc.getChild()).get(SchemaVariable.class));
	    } 	
	    if (mc.getName().equals("#method-call")) {	
		if (mc.getVariableSV() == null) {
		    return new MethodCall
			((Expression)callConvertMethod
			 (list, mc.getChild()).get(Expression.class));
		} else {
		    return new MethodCall
			(mc.getVariableSV().getSV(),(Expression)callConvertMethod
			 (list, mc.getChild()).get(Expression.class));
		}
	    } 	
	} catch (Exception e) {
	    System.err.println("Converting "+mc+" failed due to a "+e);
	    e.printStackTrace();
	} 
	throw new ConvertException("Program meta construct "
					    +mc.toString()
					    +" unknown.");	
    } 	

    protected ProgramMetaConstruct 
	convert(de.uka.ilkd.key.java.recoderext.RKeYMetaConstructExpression mc) {
	ExtList list=new ExtList();
	try {
	    if (mc.getName().equals("#static-evaluate")) {
		return new InstanceOfEval
		    ((Expression)callConvertMethod
		     (list, (recoder.java.expression.operator.Instanceof) 
		      mc.getChild()).get(Instanceof.class));	    
	    }
	    if (mc.getName().equals("#mul")) {
		return new InstanceOfEval
		    ((Expression)callConvertMethod
		     (list, (recoder.java.expression.operator.Instanceof) 
		      mc.getChild()).get(Instanceof.class));	    
	    }

	} catch (Exception e) {
	    System.err.println("Converting "+mc+" failed due to a "+e);
	    e.printStackTrace();
	} 
	throw new ConvertException("Program meta construct "
					    +mc.toString()
					    +" unknown.");	
    }


    protected ProgramMetaConstruct 
	convert(de.uka.ilkd.key.java.recoderext.RKeYMetaConstructType mc) {
	ExtList list=new ExtList();
	try {
	    if (mc.getName0().equals("#typeof")) {
		return new TypeOf
		    ((Expression)callConvertMethod
		     (list,mc.getChild()).get(Expression.class));	    
	    }
	} catch (Exception e) {
	    System.err.println("Converting "+mc+" failed due to a "+e);
	    e.printStackTrace();
	} 
	throw new ConvertException("Program meta construct "
					    +mc.toString()
					    +" unknown.");	
    }





    protected MethodCallStatement 
	convert(de.uka.ilkd.key.java.recoderext.RMethodCallStatement l) {
	ExtList list = collectChildren(l);
	return new MethodCallStatement(list, null);
    }
 

    protected ContextTerm
	convert(de.uka.ilkd.key.java.recoderext.ContextStatementBlock csb) {

	ExtList children=collectChildren(csb);
	return new ContextTerm(children);
    }


    protected SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper svw) {

	return svw.getSV();
    }


    protected SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.StatementSVWrapper svw) {

	return svw.getSV();
    }

    protected SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.LabelSVWrapper svw) {

	return svw.getSV();
    }


    protected SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.TypeSVWrapper svw) {

	return svw.getSV();
    }

    protected SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.CatchFinallySVWrapper svw) {
	return svw.getSV();
    }

    protected SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.StatementListSVWrapper slw) {
	return slw.getSV();
    }

    protected SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.ExpressionListSVWrapper elw) {
        return elw.getSV();
    }

    protected SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.ProgramVariableSVWrapper svw) {

	return svw.getSV();
    }
    

    /** parses a given JavaBlock using the context to determine the right 
     * references and returns a statement block of recoder.
     * @param block a String describing a java block
     * @param context recoder.java.CompilationUnit in which the block has 
     * to be interpreted
     * @return the parsed and resolved recoder statement block
     */
    protected recoder.java.StatementBlock 
	recoderBlock(String block, Context context) { 
	recoder.java.StatementBlock bl = null;

	SchemaJavaProgramFactory factory
	    = (SchemaJavaProgramFactory) servConf.getProgramFactory();
	factory.setSVNamespace(svns);
	try {
	    bl=factory.parseStatementBlock(new StringReader(block));
	} catch (Exception e) {
	    System.err.println("readSchemaJavaBlock(Reader,CompilationUnit)"
			       +" caused the "+
			       "exception:\n"+e);
	    e.printStackTrace();
	    throw new ConvertException(e.toString());
	}	

 	embedClass(embedMethod(embedBlock(bl), context), context);

	servConf.getChangeHistory().attached(bl);
	servConf.getChangeHistory().updateModel();

	return (recoder.java.StatementBlock)bl;
    }


    /**
     * converts a For.
     * @param f the For of recoder
     * @return the For of KeY
     */
    protected For convert(recoder.java.statement.For f) {

	ILoopInit li;
	IForUpdates ifu;
	if (f.getInitializers()!=null && 
	    f.getInitializers().getLoopInitializer(0) 
	    instanceof de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper) {
	    de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper esvw
		=(de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper)
		f.getInitializers().getLoopInitializer(0); //brrrr!

	    li=esvw.getSV();
	} else {
	    li=convertLoopInitializers(f);
	}
	if (f.getUpdates()!=null && f.getUpdates().getExpression(0) 
	    instanceof de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper) {
	    de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper esvw
		=(de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper)
		f.getUpdates().getExpression(0);
	    ifu=esvw.getSV();
	} else {
	    ifu=convertUpdates(f);
	}
	return new For(li, convertGuard(f), ifu, convertBody(f));
    }


//     protected VariableSpecification
// 	convert(recoder.java.declaration.VariableSpecification recoderVarspec){
// 	return super.convert(recoderVarspec);
//     }


     protected VariableSpecification convertVarSpecWithSVType
 	(recoder.java.declaration.VariableSpecification recoderVarspec){
	 VariableSpecification varspec=
 	    (VariableSpecification) rec2key().toKeY(recoderVarspec);
 	if (varspec==null) {	    
 	    ExtList l=collectChildren(recoderVarspec);	    
	    ProgramElement pv
		= ProgramSVSort.VARIABLE.getSVWithSort(l, 
						       ProgramElementName.class);
	    if (pv instanceof ProgramElementName) {  //sth. like #type i;
		KeYJavaType kjt=new KeYJavaType(typeSVType);
		pv=new ProgramVariable((ProgramElementName)pv, kjt);
	    }
	    ProgramElement init
		= ProgramSVSort.VARIABLEINIT.getSVWithSort
		(l, Expression.class);
 	    varspec = new VariableSpecification((IProgramVariable)pv, 
						recoderVarspec.getDimensions(),
						(Expression)init,
						typeSVType);
 	    insertToMap(recoderVarspec, varspec);
 	}
 	return varspec;
     }

     protected LocalVariableDeclaration
 	convert(recoder.java.declaration.LocalVariableDeclaration lvd) {  
 	if (lvd.getTypeReference() instanceof TypeSVWrapper) {
 	    recoder.list.VariableSpecificationList rspecs = lvd.getVariables();
 	    VariableSpecification[] varspecs
 		= new VariableSpecification[rspecs.size()];
 	    for (int i=0; i<rspecs.size(); i++) {
 		varspecs[i] = convertVarSpecWithSVType
 		    (rspecs.getVariableSpecification(i));
 	    }
 	    SchemaVariable typesv
 		= ((TypeSVWrapper)lvd.getTypeReference()).getSV();
 	    return new LocalVariableDeclaration(new Modifier[0], typesv, varspecs);
 	} else {
 	    return super.convert(lvd);
 	}
     }



     /** convert a recoder VariableSpecification to a KeY
      * VariableSpecification
      * (checks dimension and hands it over and insert in hashmap)
      */
     protected VariableSpecification
 	convert(recoder.java.declaration.VariableSpecification recoderVarspec){	

 	if (! (recoderVarspec.getIdentifier() instanceof ProgramVariableSVWrapper)) {
 	    return super.convert(recoderVarspec);
 	}
 	VariableSpecification varSpec
 	    = (VariableSpecification)rec2key().toKeY(recoderVarspec);
 	if (varSpec==null) {

 	    ProgramElementName name=new ProgramElementName
 		(recoderVarspec.getName());
 	    ExtList children = collectChildren(recoderVarspec);
 	    IProgramVariable progvar 
		= (IProgramVariable) children.get(SchemaVariable.class);

	    children.remove(progvar);
 	    varSpec = new VariableSpecification(children, 
 					    progvar,
 					    recoderVarspec.getDimensions(),
 					    null);
 	    insertToMap(recoderVarspec, varSpec);

 	}
 	return varSpec;
     }


    //-------------------------------------------------------------------------
    // What follows are just the signatures of the more special convert methods
    // of the super class. As we use reflection to determine the appropriate
    // convert method and Java gets confused when this subclass is involved
    // (or at least I do ;-) during getDeclaredMethod, we repeat the signatures 
    // here and call the respective methods of Recoder2KeY.
    //-------------------------------------------------------------------------

    protected ProgramElement convert(recoder.java.ProgramElement pe) {
	return super.convert(pe);
    }
    
    protected NewArray convert(recoder.java.expression.operator.NewArray newArr){
	return super.convert(newArr);
    }

    protected IntLiteral convert(recoder.java.expression.literal.IntLiteral 
			       intLit) {
	return super.convert(intLit);
    }
    
    protected BooleanLiteral 
	convert(recoder.java.expression.literal.BooleanLiteral booleanLit) {
	return super.convert(booleanLit);
    }

    protected StringLiteral 
	convert(recoder.java.expression.literal.StringLiteral stringLit) {
	return super.convert(stringLit);
    }

    protected DoubleLiteral 
	convert(recoder.java.expression.literal.DoubleLiteral doubleLit) {
	return super.convert(doubleLit);
    }

    protected FloatLiteral 
	convert(recoder.java.expression.literal.FloatLiteral floatLit) {
	return super.convert(floatLit);
    }

    protected LongLiteral 
	convert(recoder.java.expression.literal.LongLiteral longLit) {
	return super.convert(longLit);
    }

    protected CharLiteral 
	convert(recoder.java.expression.literal.CharLiteral charLit) {
	return super.convert(charLit);
    }

    protected NullLiteral 
	convert(recoder.java.expression.literal.NullLiteral nullLit) {
	return super.convert(nullLit);
    }
    protected EmptyStatement convert(recoder.java.statement.EmptyStatement
				       eStmnt) {	
	return super.convert(eStmnt);
    }
    protected ProgramElementName convert(recoder.java.Identifier id) {
	return super.convert(id);
    }

    protected ProgramMethod
	convert(recoder.java.declaration.MethodDeclaration md) {     
	return super.convert(md);
    }


    protected FieldSpecification
 	convert(recoder.java.declaration.FieldSpecification recoderFieldspec){
	return super.convert(recoderFieldspec);
    }


    protected ParameterDeclaration
	convert(recoder.java.declaration.ParameterDeclaration pd) {    
	return super.convert(pd);
    }

    protected FieldDeclaration
 	convert(recoder.java.declaration.FieldDeclaration fd) {    
	return super.convert(fd);
     }

    protected ConstructorDeclaration
 	convert(recoder.java.declaration.ConstructorDeclaration cd) {    
	return super.convert(cd);
     }


    protected TypeReference
	convert(recoder.java.reference.TypeReference tr) {    
	return super.convert(tr);
    }

    protected JavaProgramElement 
	convert(recoder.java.reference.UncollatedReferenceQualifier urq) {    
	return super.convert(urq);
    }

    protected ProgramVariable convert
	 (recoder.java.reference.VariableReference vr) {
	return super.convert(vr);
    }

    protected FieldReference convert
	 (recoder.java.reference.FieldReference fr) {
	return super.convert(fr);
    }

    protected MethodReference convert
	 (recoder.java.reference.MethodReference mr) {
	return super.convert(mr);
     }

    protected While convert(recoder.java.statement.While w) {
	return super.convert(w);
    }

    protected Do convert(recoder.java.statement.Do d) {
	return super.convert(d);
    }


    protected ArrayReference convert(recoder.java.reference.ArrayReference ar) {
	return super.convert(ar);
    }

    protected SuperConstructorReference 
	convert(recoder.java.reference.SuperConstructorReference scr) {
	return super.convert(scr);
    }

    protected Case 
	convert(recoder.java.statement.Case c) {
	return super.convert(c);
    }

    protected LabeledStatement 
	convert(recoder.java.statement.LabeledStatement l) {
	return super.convert(l);
    }

    protected New 
	convert(recoder.java.expression.operator.New n) {
	return super.convert(n);
    }


    protected Import convert(recoder.java.Import im) {
	return super.convert(im);
    }

    protected Instanceof convert
	(recoder.java.expression.operator.Instanceof rio) {       
	return super.convert(rio);
    }

    protected TypeCast convert
	(recoder.java.expression.operator.TypeCast c) {       
	return super.convert(c);
    }


} 
