// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/**
 * A ProofObligationTerm is an object that contains an Operator and one rulevariable
 * Instances should never be accessed via 
 * this interface, use the interface of the superclass Term and construct instances only via
 * a TermFactory instead.
 */

package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.rule.Rulevariable;

class ProofObligationTerm extends Term {
    

    private final Rulevariable ruleVar;

       
    /** creates a proofobligation term
     * @param ruleVar the rulevariable for which the term represents 
     * it's proofobligation.
     * @param op - should be the proofobl operator
     */
    public ProofObligationTerm(Rulevariable ruleVar) {
	super(Op.PROOFOBL);
	this.ruleVar=ruleVar;
	setSort();
	determineFreeVars();
    }

    /** @return null for all n (no subterms !)  */    
    public Term sub(int n) {
	return (null); // we have no subterms ...
    }	
   
    /** @return arity of the proofobligation term  as int */
    public int arity() {
	return 0;
    } 

    /**@return depth of the term. Always 0*/
    public int depth() {
	return 0;
    }

    /** @return the bound variables of the n-th subterm */
    public ArrayOfQuantifiableVariable varsBoundHere(int n) {
	return (EMPTY_VAR_LIST);
    }
    
    /** @return rulevariable if term is proofobligation term  */
    public Rulevariable ruleVar() {
	return this.ruleVar;
    }
   

}
