// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic;

import junit.framework.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.op.*;

public class TestSemisequent extends TestCase {
 
    private ConstrainedFormula[] con;

    public TestSemisequent(String name) {
	super(name);
    }


    public void setUp() { 
	TermFactory tf=TermFactory.DEFAULT;
	
       	Function p=new Function(new Name("p"),Sort.FORMULA,new Sort[]{});  
	Function q=new Function(new Name("q"),Sort.FORMULA,new Sort[]{});
	Function r=new Function(new Name("r"),Sort.FORMULA,new Sort[]{});
	Term t_p=tf.createFunctionTerm(p, new Term[]{});
	Term t_q=tf.createFunctionTerm(q, new Term[]{});
	Term t_r=tf.createFunctionTerm(r, new Term[]{});
	
	con=new ConstrainedFormula[4];
	con[0]=new ConstrainedFormula(t_p, Constraint.BOTTOM);
	con[1]=new ConstrainedFormula(t_q, Constraint.BOTTOM);
	con[2]=new ConstrainedFormula(t_r, Constraint.BOTTOM);
	con[3]=new ConstrainedFormula(t_r, Constraint.BOTTOM);
    }
       
    public void testindexOf() {
	Semisequent seq=Semisequent.EMPTY_SEMISEQUENT;
	seq=seq.insert(0,con[0]);
	seq=seq.insert(1,con[1]);
	seq=seq.insert(2,con[2]);
	assert("Semisequent has wrong size.", seq.size()==3);
	assert("con[0] has wrong index in semisequent. Expected 0"+
	       " has "+ seq.indexOf(con[0]), seq.indexOf(con[0])==0); 
	assert("con[1] has wrong index in semisequent. Expected 1"+ 
	       " has "+seq.indexOf(con[1]), seq.indexOf(con[1])==1);
	assert("con[2] has wrong index in semisequent. Expected 2"+ 
	       " has "+seq.indexOf(con[2]), seq.indexOf(con[2])==2);	
    }

    public void testRemove() {

	Semisequent seq=Semisequent.EMPTY_SEMISEQUENT;
	seq=seq.insert(0,con[0]);
	seq=seq.insert(1,con[1]);
	seq=seq.insert(2,con[2]);
	seq=seq.remove(1);
	
	assert("Semisequent has wrong size.", seq.size()==2);
	assert("Semisequent contains deleted element.",!seq.contains(con[1]));
	assert("con[1] has wrong index in semisequent",seq.indexOf(con[1])==-1);
	assert("con[0] has wrong index in semisequent",seq.indexOf(con[0])==0);
	assert("con[2] has wrong index in semisequent",seq.indexOf(con[2])==1);		
    }

    public void testReplace() {
	Semisequent seq=Semisequent.EMPTY_SEMISEQUENT;
	seq=seq.insert(0,con[0]);
	seq=seq.insert(1,con[1]);	
	seq=seq.replace(1,con[2]);
	
	assert("Semisequent has wrong size.", seq.size()==2);
	assert("Semisequent contains deleted element.",!seq.contains(con[1]));
	assert("con[1] has wrong index in semisequent",seq.indexOf(con[1])==-1);
	assert("con[0] has wrong index in semisequent",seq.indexOf(con[0])==0);
	assert("con[2] has wrong index in semisequent",seq.indexOf(con[2])==1);		
    }

    public void testNoDuplicates() {
	Semisequent seq=Semisequent.EMPTY_SEMISEQUENT;
	seq=seq.insert(0,con[0]);
	seq=seq.insert(1,con[1]);
	seq=seq.insert(2,con[1]);
	
	assert("Semisequent has duplicate",seq.size()==2);		
    }


    public void testImmutable() {
	Semisequent seq=Semisequent.EMPTY_SEMISEQUENT;
	Semisequent old=Semisequent.EMPTY_SEMISEQUENT;
	seq=seq.insert(0,con[0]);
	seq=seq.insert(1,con[1]);
	old=seq;
	seq=seq.insert(2,con[2]);
	
	assert("Semisequent seems not to be immutable.",old.size()==2);
    }

    public void testUniqueEmpty() {
	Semisequent seq=Semisequent.EMPTY_SEMISEQUENT;
	seq=seq.insert(0,con[0]);
	seq=seq.remove(0);	
	assert("Semisequent is empty but not the EMPTY_SEMISEQUENT.",seq==Semisequent.EMPTY_SEMISEQUENT);

    }

    public void testEquals() {
	Semisequent mt=Semisequent.EMPTY_SEMISEQUENT;
	Semisequent seq1 = Semisequent.EMPTY_SEMISEQUENT;
	seq1 = seq1.insert(0,con[0]);
	seq1 = seq1.insert(0,con[1]);
	seq1 = seq1.insert(0,con[2]);
	Semisequent seq2 = Semisequent.EMPTY_SEMISEQUENT;
	seq2 = seq2.insert(0,con[0]);
	seq2 = seq2.insert(0,con[1]);
	seq2 = seq2.insert(0,con[2]);
	Semisequent seq3 = Semisequent.EMPTY_SEMISEQUENT;
	seq3 = seq3.insert(0,con[0]);
	seq3 = seq3.insert(0,con[1]);
	seq3 = seq3.insert(0,con[3]);
	Semisequent seq4 = Semisequent.EMPTY_SEMISEQUENT;
	seq4 = seq4.insert(0,con[0]);
	seq4 = seq4.insert(0,con[2]);
	seq4 = seq4.insert(0,con[1]);
	
	assertEquals("seq1=seq1",seq1,seq1);
	assertEquals("seq1=seq2",seq1,seq2);
	assertEquals("seq1=seq3",seq1,seq3);
	assert("seq1!=seq4", ! seq1.equals(seq4));
    }

}
