// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.logic.op.Operator;
import de.uka.ilkd.key.logic.op.ArrayOfQuantifiableVariable;
/**
 * An UpdateTerm is a term attached with an update indicating that the
 * value of a program variable has changed. 
 */
class UpdateTerm extends Term {

    /** the term that updates the programvariable of the operator */
    private final Term updateTerm;    

    /** the term that is attached with the update */
    private final Term term;    

    /** depth of the term */
    private final int depth ;

    /** creates an UpdateTerm
     * @param op the UpdateOperator
     * @param term the Term that is updated
     * @param updateTerm the Term describing the value of the
     * programvariable in the update operator
     */
    public UpdateTerm(Operator op, 
		      Term updateTerm, 
		      Term term) {
	super(op);
	this.updateTerm = updateTerm;
	this.term  = term;
	depth = term.depth() + 1;
	setSort();
	determineFreeVars();
    }

    /** @return the term being updated  */    
    public Term sub(int n) {
	return n == 0 ? updateTerm : term;
    }
   
    /** @return arity of the update term 1 as int */
    public int arity() {
	return 2;
    } 

    /**@return depth of the term */
    public int depth() {
	return depth;
    }

    /** @return the bound variables of the n-th subterm */
    public ArrayOfQuantifiableVariable varsBoundHere(int n) {
	return EMPTY_VAR_LIST;
    }

    public String toString() {
	return op()+updateTerm.toString()+"}"+term.toString();
    }
}
