// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.sort.ObjectSort;
import de.uka.ilkd.key.logic.Term;

/**
 * This is a `conditional term' operator (T0 ?= T1).a /-> T1 . 
 */
public class CondTermOp extends Op {

    private final AttributeOp attributeOp;


    public CondTermOp(AttributeOp attrOp) {
	super(attrOp.attribute().name());
	this.attributeOp = attrOp;
    }

    /**
     * @return true iff the arity equals 3 and the sort of the subterm 0 
     * equals sort of subterm 1
     */
    public boolean validTopLevel(Term term){
	if (term.arity() != 3) return false;
	return ((term.sub(0).sort() instanceof ObjectSort) 
		&& (term.sub(1).sort() instanceof ObjectSort)) ||
	    term.sub(0).sort() == term.sub(1).sort();
    }

    /**
     * @return sort of the sub(0) subterm or throws an
     * IllegalArgumentException if the given term has no correct (3=) arity
     */
    public Sort sort(Term term) {
	if (term.arity()==3)
	    return attributeOp.attribute().sort();
	else throw new IllegalArgumentException("Cannot determine sort of "+
						"invalid term (Wrong arity).");
    }

    public AttributeOp attribute() {
	return attributeOp;
    }

    /** @return arity of the Substitution operator as int */
    public int arity() {
	return 3;
    }
}
