// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/* Generated by Together */

package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
public class SubstOp extends Op {
    
    public SubstOp(Name name) {
	super(name);
    }

    /**
     * @return true iff the sort of the subterm 0 of the given term
     * has the same sort as or a subsort of the variable to be substituted 
     * and the
     * term's arity is 2 and the numer of variables bound there is 0
     * for the 0th subterm and 1 for the 1st subterm.
     */
    public boolean validTopLevel(Term term){
	if (term.arity() != 2) return false;
	if (term.varsBoundHere(1).size() != 1) return false;
	if (term.varsBoundHere(0).size() != 0) return false;
	Sort substSort=term.sub(0).sort();
	Sort varSort=term.varsBoundHere(1).getQuantifiableVariable(0).sort();
	return substSort.extendsTrans(varSort);
    }

    /**
     * @return sort of the sub(1) subterm or throws an
     * IllegalArgumentException if the given term has no correct (2=) arity
     */
    public Sort sort(Term term) {
	if (term.arity()==2)
	    return term.sub(1).sort();
	else throw new IllegalArgumentException("Cannot determine sort of "+
						"invalid term (Wrong arity).");
    }

    /** @return arity of the Substitution operator as int */
    public int arity() {
	return 2;
    }
}
