// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.Name;
public class UpdateOp implements Operator {

    /** the variable that is updated */
    private final Term updatedVar;
    /** name of this operator */
    protected final Name name = new Name("update");


    public UpdateOp(Term updatedVar) {
	if (!(updatedVar.op() instanceof SchemaVariable  || 
	      updatedVar.op() instanceof ProgramVariable ||
	      updatedVar.op() instanceof AttributeOp)) {
	    throw new 
		IllegalArgumentException("Only variables or attributes "+
					 "can be updated not "+updatedVar.op()+
					 " of class "+
					 updatedVar.op().getClass());
	}
	this.updatedVar = updatedVar;
    }


    /** @return name of the Operator */
    public Name name() {
	return name;
    }

    /** @return true if the given term has arity 2
     */
    public boolean validTopLevel(Term term){
	if (term.arity() != 2) return false;
	return true;
    }

    /** @return arity of the update operator as int */
    public int arity() {
	return 2;
    }

    /** @return true if same object or the update variables/attributes 
 * 	are the same
    */
    public boolean equals(Object o) {
	if (!(o instanceof UpdateOp) || o == null) {
	    return false;
	}
	if (o == this) {
	    return true;	    
	}
	return updatedVar.equals(((UpdateOp)o).updatedVar);
    }

    /** @return sort of the term
     */
     public Sort sort(Term term) {
        return term.sub(1).sort();
    }  

    /** @return the variable being updated */
    public Term variable() {
	return updatedVar;
    }

    /** toString */
    public String toString() {
	return "{"+variable()+":=";
    }

}
