// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001 Universitaet Karlsruhe, Germany
//                    and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General 
// Public License. See LICENSE.TXT for details.
//


package de.uka.ilkd.key.ocl.tudresden;

// attention: do not import java.util.Collection
// The tudresden-package contains a class with the same name
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.io.*;

import tudresden.ocl.check.types.*;
import tudresden.ocl.check.OclTypeException;
import com.togethersoft.openapi.sci.*;
import com.togethersoft.openapi.sci.enum.*;
import com.togethersoft.openapi.rwi.*;
import com.togethersoft.openapi.rwi.enum.*;

import de.uka.ilkd.key.ocl.tudresden.*;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.recoderext.*;

import recoder.service.*;
import recoder.abstraction.Package;
import recoder.list.*;
import recoder.convenience.*;
import recoder.io.*;
import recoder.util.*;
import recoder.java.*;

import com.togethersoft.openapi.ide.IdeAccess;
import com.togethersoft.openapi.ide.project.*;

/**
* This class reads in the UML-model of a collection of java source files.
* It does this using recoder instead of reflection like the JavaModelFacade
* of the dresdenocl-package.
* For reading in associations it relies on the API of Together ControlCenter
* as the CASE-tool in which the model (should) has been created.
* @author Steffen Schlager, Hans-Joachim Daniels
*/
public class TJModelFacade extends JavaOCLTypeMap implements ModelFacade{

    HashMap classes;
    HashMap recoderclasses;
    protected String containingPackage;

    public HashMap getClasses() 
    {
	return classes;
    }
    
    public TJModelFacade(String cPack, String projectRoot){
	if (cPack!=null)
	    containingPackage = cPack;
	else
	    containingPackage = "";
	classes=new HashMap();
	recoderclasses=new HashMap();
	String nameString = new String();
	
        /* To get the primitive types in recoder the JavaProgramFactory has
	 * to be instantiated
         */
	
        String dirString = projectRoot;
	Services s = new Services();
	Recoder2KeY r2k = new Recoder2KeY(dirString, s.getJavaInfo().rec2key(), 
					  s.getTypeConverter());
	KeYCrossReferenceServiceConfiguration servConf = r2k.getServiceConfiguration();
	SourceFileRepository sfr = servConf.getSourceFileRepository();
        DefaultSourceInfo sfi = new DefaultSourceInfo(servConf);
	String[] files = collectJavaFiles(dirString);
	try{
	    for (int i=0; i<files.length; i++) {
                if (files[i] != null) {
		    sfr.getCompilationUnitFromFile(files[i]);
                }
            }
	} catch (Exception e){
            System.out.println("  Exception occured while reading in Java sources: " + e);
        }
	/* This loop has to be executed twice, to look what classes are found and
	   defined in the diagram (todo: look up the classes in the Together
	   diagram) and then to analyse them, just like before.
	   The same loop because recoder's powers have to be used which only
	   works with the used iterator.
	*/
	CompilationUnitList nameInfo = sfr.getCompilationUnits();
	
	for (int i=0; i < nameInfo.size(); i++) {
	    recoder.java.CompilationUnit pa = nameInfo.getCompilationUnit(i);
	    //			int anzClasses = pa.getTypeDeclarationCount();
	    for (int j = 0; j < pa.getTypeDeclarationCount(); j++) {
		recoder.java.declaration.TypeDeclaration td = pa.getTypeDeclarationAt(j);		
		TJClassifier tjc = new TJClassifier(td.getFullName(), td.getFullName(), recoderclasses);
		classes.put(td.getFullName(),tjc);
		recoderclasses.put(td.getFullName(),td);
		//System.out.println("Fge ein: "+td.getFullName());

		if (td.getPackage().getFullName().equals(containingPackage)) {
		    tjc = new TJClassifier(td.getName(), td.getFullName(), recoderclasses);
		    classes.put(td.getName(),tjc);
		    recoderclasses.put(td.getName(),td);
		    //System.out.println("Fge ein: "+td.getName());
		} 
	    }
	}
	
	for (int i=0; i < nameInfo.size(); i++) {
	    recoder.java.CompilationUnit pa = nameInfo.getCompilationUnit(i);
	    for (int j = 0; j < pa.getTypeDeclarationCount(); j++) {
		recoder.java.declaration.TypeDeclaration td = pa.getTypeDeclarationAt(j);
		boolean inCurrentPackage = td.getPackage().getFullName().equals(containingPackage);
		TJClassifier tjc2=null;
		if (inCurrentPackage)
		    tjc2 = (TJClassifier)classes.get(td.getName());
		TJClassifier tjc=(TJClassifier)classes.get(td.getFullName());
		FieldList fl = td.getAllFields();
		for (int l=0; l < fl.size(); l++) {
		    recoder.abstraction.Field field;
		    field = fl.getField(l);
		    tjc.features.put(field.getName(), new TJStructuralFeature
			(field.getName(), getTypeFor(field.getType(), classes)));
		    //		    System.out.println("Klasse: "+tjc+" enthlt Feld: "+field.getName());
		    if (inCurrentPackage) {
			tjc2.features.put(field.getName(), new TJStructuralFeature
			    (field.getName(), getTypeFor(field.getType(), classes)));
			//			System.out.println("Klasse: "+tjc2+" enthlt Feld: "+field.getName());
		    }
		}
		
		MethodList ml = td.getAllMethods();
		for (int l=0; l < ml.size(); l++) {
		    recoder.abstraction.Method meth;
		    meth = ml.getMethod(l);
		    recoder.abstraction.Type retType = meth.getReturnType();
		    tudresden.ocl.check.types.Type ret2 = getTypeFor(meth.getReturnType(), classes);
		    
		    nameString = new String(meth.getName() + "(");
		    TypeList ctl = meth.getSignature();
		    Type[] paramsArray = new Type[ctl.size()];
		    for (int k=0; k < ctl.size(); k++) {

			recoder.abstraction.Type par;
			par = ctl.getType(k);
                        if (k < ctl.size() - 1) {
			    nameString = nameString + par.getFullName() + ",";
                        } else {
                            nameString = nameString + par.getFullName();
                        }
			paramsArray[k] = getTypeFor(par, classes);
		    }
		    nameString = nameString + ")";
		    
		    if (retType!=null) {
			tjc.features.put(nameString ,new TJBehaviouralFeature(nameString,ret2,paramsArray));
			if (inCurrentPackage){
			    tjc2.features.put(nameString ,new TJBehaviouralFeature(nameString,ret2,paramsArray));
			}
			    
		    } else {
			tjc.features.put(nameString ,new TJBehaviouralFeature(nameString,paramsArray));
			if (inCurrentPackage){
			    tjc2.features.put(nameString ,new TJBehaviouralFeature(nameString,ret2,paramsArray));
			}
			
		    }
		}
	    }
	}
			
	SciModel model = SciModelAccess.getModel();
	
	// find associations
	RwiModel rwiModel = RwiModelAccess.getModel();		
	RwiPackageEnumeration rwiRoots=rwiModel.
	    rootPackages(RwiProperty.MODEL);
	while (rwiRoots.hasMoreElements()) {
	    RwiPackage rwiPackage = rwiRoots.nextRwiPackage();
	    processAssociations(rwiPackage);
	}

	// now we have to go through the list of all classes to
	// let subclasses inherit the associations of the superclass
	TJClassifier tjcSuper, tjcSub;
	for (int i=0; i < nameInfo.size(); i++) {	
	    recoder.java.CompilationUnit pa = nameInfo.getCompilationUnit(i);
	    for (int j = 0; j < pa.getTypeDeclarationCount(); j++) {
		
		recoder.java.declaration.TypeDeclaration td = pa.getTypeDeclarationAt(j);
		ClassTypeList supertypes = td.getAllSupertypes();

		for (int l = 0; l < supertypes.size(); l++) {
		    try{
			tjcSuper = (TJClassifier)classes.get(((recoder.java.declaration.TypeDeclaration)
							      supertypes.getClassType(l)).getFullName());
			tjcSub = (TJClassifier)classes.get(td.getFullName());
			//			System.out.println("superclass: "+tjcSuper+" subclass: "+tjcSub);
 			if (tjcSuper!=null && tjcSuper.assocEnds!=null)
			    tjcSub.assocEnds.putAll(tjcSuper.assocEnds);

			boolean inCurrentPackage = td.getPackage().getFullName().equals(containingPackage);
			//			System.out.println(td.getPackage().getFullName()+" : "+containingPackage);
			if (inCurrentPackage){
			    tjcSub = (TJClassifier)classes.get(td.getName());
			    if (tjcSuper!=null && tjcSuper.assocEnds!=null)
				tjcSub.assocEnds.putAll(tjcSuper.assocEnds);
			}
		    } catch (Exception e){}
		}
	    }
	}
	
	//	System.out.println("--------------------- Auslesen des Modells beendet!");
    }

    protected void processAssociations(RwiPackage rwiPackage){
	findAssociations(rwiPackage);
	RwiPackageEnumeration subpackages = rwiPackage.subpackages();
	while (subpackages.hasMoreElements()) {
	    RwiPackage subpackage = subpackages.nextRwiPackage();
	    processAssociations(subpackage);
	}
    }

    protected void findAssociations(RwiPackage pack) {
	SciModel model = SciModelAccess.getModel();
	RwiNodeEnumeration rwiNodesEnum = pack.nodes();
	while (rwiNodesEnum.hasMoreElements()){
	    RwiNode rwiNode = rwiNodesEnum.nextRwiNode();
	    //System.out.println("    RwiNode: "+rwiNode);
	    RwiLinkEnumeration linkEnum = rwiNode.outgoingLinks();
	    while (linkEnum.hasMoreElements()){
		RwiLink link = linkEnum.nextRwiLink();
		//filter out associations
		if (link.getProperty(RwiProperty.SHAPE_TYPE).
		    equals(RwiShapeType.ASSOCIATION)){
		    // get cardinality and client role
		    Multiplicity mult=null;
		    String clientRole=null;
		    RwiMemberEnumeration rwiMemberEnum = 
			rwiNode.members();
		    RwiMember rwiMember=null;
		    while (rwiMemberEnum.hasMoreElements()){
			rwiMember = rwiMemberEnum.nextRwiMember();
			if (rwiMember.getUniqueName().indexOf(link.toString())!=-1){
			    SciElement e = model.findElement(rwiMember.getUniqueName());
			    String clientCard = model.findElement
				(rwiMember.getUniqueName()).getTagList().
				getTagValue("supplierCardinality");
			    if (clientCard!=null){
				if (clientCard.equals("0..*") || 
				    clientCard.equals("*")){
				    mult=Multiplicity.ZERO_TO_INF;
				} else if (clientCard.equals("1..*")){
				    mult=Multiplicity.ONE_TO_INF;
				} else if (clientCard.equals("1")){
				    mult=Multiplicity.STANDARD;
				} else if (clientCard.equals("0..1")){
				    mult=Multiplicity.ZERO_OR_ONE;
				}
			    } else
				mult=Multiplicity.STANDARD;
			    clientRole = model.findElement
				(rwiMember.getUniqueName()).getTagList().
				getTagValue("supplierRole");
			    if (clientRole==null){
				clientRole=link.getDestination().toString();
				clientRole=clientRole.substring(0,1).toLowerCase()+
				    clientRole.substring(1,clientRole.length());
			    } 					    
// 			    System.out.println("    Client Role: "+clientRole+
// 					       " clientCard: "+mult);
			    break;
			}
		    }
		    //		    System.out.println(rwiNode.getContainingPackage());
		    String prefix = getContainingPackage(rwiNode.getContainingPackage());
		    String key = prefix.equals("") ? rwiNode.toString() : prefix+rwiNode;
		    prefix = getContainingPackage
			(((RwiContainer)link.getDestination()).getContainingPackage());
		    String targetClass = prefix.equals("") ? link.getDestination().toString() :
			prefix+link.getDestination();
		    //		    System.out.println("--> Assoiation von "+key+" nach "+targetClass);			    
		    if (classes.containsKey(targetClass)) {
			TJClassifier linkTarget = (TJClassifier)classes.get(targetClass);
			((TJClassifier)classes.get(key)).
			    assocEnds.put(clientRole, new TJAssocEnd
				(clientRole, linkTarget,mult));
			//				System.out.println("Fge Assoziation ein von "+key+" nach "+targetClass+
				//	   ". Client-Rolle ist "+clientRole);
			if (getContainingPackage(rwiNode.getContainingPackage()).equals(containingPackage)) {
			    ((TJClassifier)classes.get(rwiNode.toString())).
				assocEnds.put(clientRole, new TJAssocEnd
				    (clientRole, linkTarget, mult));

			    //  System.out.println("Fge Assoziation ein von "+rwiNode+" nach "+targetClass+
			    //		       ". Client-Rolle ist "+clientRole);
			}
		    }
		    // associations are bi-directional. So we have to add the association also to
		    // the target classifier 
		    // -------------------------------------
		    mult=null;
		    clientRole=null;

		    SciElement e = model.findElement(rwiNode.getUniqueName());
		    String clientCard = model.findElement
			(rwiNode.getUniqueName()).getTagList().
			getTagValue("clientCardinality");
		    if (clientCard!=null){
			if (clientCard.equals("0..*") || 
			    clientCard.equals("*")){
			    mult=Multiplicity.ZERO_TO_INF;
			} else if (clientCard.equals("1..*")){
			    mult=Multiplicity.ONE_TO_INF;
			} else if (clientCard.equals("1")){
			    mult=Multiplicity.STANDARD;
			} else if (clientCard.equals("0..1")){
			    mult=Multiplicity.ZERO_OR_ONE;
			}
		    } else
			mult=Multiplicity.STANDARD;
		    clientRole = model.findElement
			(rwiMember.getUniqueName()).getTagList().
			getTagValue("clientRole");
		    if (clientRole==null){
			clientRole=rwiNode.toString();
			clientRole=clientRole.substring(0,1).toLowerCase()+
			    clientRole.substring(1,clientRole.length());
		    } 					    
		    // 		    System.out.println("    Client Role: "+clientRole+
		    //	       " clientCard: "+mult);
		    // 		    System.out.println(rwiNode.getContainingPackage());
 		    //System.out.println("--> Assoiation von "+targetClass+" nach "+key);			    
		    if (classes.containsKey(key) && classes.containsKey(targetClass)) {
			TJClassifier linkTarget = (TJClassifier)classes.get(key);
			((TJClassifier)classes.get(targetClass)).
			    assocEnds.put(clientRole, new TJAssocEnd
				(clientRole, linkTarget,mult));
// 			System.out.println("Fge Assoziation ein von "+targetClass+" nach "+key+
// 					   ". Client-Rolle ist "+clientRole);
 			if (getContainingPackage(rwiNode.getContainingPackage()).equals(containingPackage)) {
 			    ((TJClassifier)classes.get(link.getDestination().toString())).
 				assocEnds.put(clientRole, new TJAssocEnd
 				    (clientRole, linkTarget, mult));
// 			    System.out.println("Fge Assoziation ein von "+
// 					       link.getDestination().toString()+" nach "+linkTarget+
// 			    		       ". Client-Rolle ist "+clientRole);
			}
		    }
		    // -------------------------------------


		}
	    }
	}
    }

    protected String getContainingPackage(RwiContainer arg){
	String result="";

	while (arg.getContainingPackage() != null &&
	       ! arg.getContainingPackage().equals("<default>")) {
	    result=arg+"."+result;
	    arg=arg.getContainingPackage();
	}

	if (result.startsWith("."))
	    result=result.substring(1,result.length());
	return result;
    }
    
    
    protected boolean isInAnotherPackage(String name){
	Iterator classIt = classes.entrySet().iterator();
	String next;
	while (classIt.hasNext()){
	    next = classIt.next().toString();
	    if (next.indexOf(name)!=-1) {
		return true;
	    }
	}
	return false;
    }
    
    /** @param name the name of an application type (not the name of a predefined
     *              type, like &quot;Integer&quot;), possibly a "::"-separated
     *              path name
     *  @throws OclTypeException if no type with the given name exists
     */
    public Any getClassifier(String name) {
	//	System.out.println("  getClassifier aufgerufen fr :"+name);
        String nameString = new String();
        int k = name.indexOf(" :: ");
        int oldk = 0;
        while (k != -1) {
	    nameString += name.substring(oldk, k) + ".";
	    oldk = k + 4;
            k = name.indexOf(" :: ", oldk);
        }
        nameString += name.substring(oldk);
	if (classes.containsKey(nameString)){
	    //	    System.out.println("    Gebe zurck: "+(TJClassifier)classes.get(nameString));
	    return (TJClassifier)classes.get(nameString);
	}
	else {		    
	    if (isInAnotherPackage(nameString)) {		       
		//		System.out.println("     Klasse "+name+" existiert in anderem Package.");
		return null;
	    }
	    else
		throw new OclTypeException
		    ("can not find classifier \""+name+"\"");
	}
    }
    

    private static String[] collectJavaFiles(String dir) {
	//        System.out.println("  dir: " + dir);
	FileCollector col = new FileCollector(dir);
	StringMutableList list = new StringArrayList();
	while (col.next(DefaultSourceFileRepository.JAVA_FILENAME_FILTER)) {
	    String path;
	    try {
		path = col.getFile().getCanonicalPath();
	    } catch (IOException ioe) {
		path = col.getFile().getAbsolutePath();
	    }
	    list.add(path);
	    //            System.out.println("  " + path + col.getFile().getName());
	}
	String res[]=new String[1000];
	try{
	    int i=0;
	    while (true){
		res[i]=list.getString(i);
		i++;
	    }
	} catch (Exception e){}
	return res;
    }

    
    public String toString(){
	return getClass().getName();
    }
    
}
