// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//


package de.uka.ilkd.key.proof;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.util.Debug;


/** Table model storing a raw constraint as a vector of equalities
 * (term pairs) */

public class ConstraintTableModel extends AbstractTableModel {

    /** Vector of term pairs representing equalities */
    protected Vector      entries           = new Vector ();

    /** This constraint normalized */
    protected Constraint  currentConstraint = Constraint.BOTTOM;

    public ConstraintTableModel () {}

    public Constraint getConstraint () {
	return currentConstraint;
    }

    /** Methods from AbstractTableModel */

    public int getRowCount () {
	return entries.size ();
    }

    public int getColumnCount () {
	return 2;
    }

    public Object getValueAt ( int p_row, int p_column ) {
	if ( p_column == 0 )
	    return ((PairOfTerm)entries.get ( p_row )).left;
	else
	    return ((PairOfTerm)entries.get ( p_row )).right;
    }

    public String getColumnName ( int p_column ) {
	if ( p_column == 0 )
	    return "Left";
	else
	    return "Right";
    }

    /** Add a new equality to this constraint, consisting of a pair of
     * terms with the same sort (not Sort.FORMULA)
     * @param p_left left side of the equality
     * @param p_right right side of the equality
     */
    public void addEquality ( Term p_left, Term p_right ) {
	Debug.assert ( p_left.sort () == p_right.sort () && p_left.sort () != Sort.FORMULA,
		       "Terms have different or wrong sort" );

	entries.add                                 ( new PairOfTerm ( p_left, p_right ) );
	currentConstraint = currentConstraint.unify ( p_left, p_right );
	fireTableRowsInserted                       ( entries.size (), entries.size () );	
    }

    /** Delete one equality
     * @param p_row number of the equality, has to be within [0..getRowCount())
     */
    public void deleteRow ( int p_row ) {
	entries.remove       ( p_row );

	currentConstraint = Constraint.BOTTOM;
	Iterator   it     = entries.iterator ();
	PairOfTerm p;

	while ( it.hasNext () ) {
	    p                 = (PairOfTerm)it.next ();
	    currentConstraint = currentConstraint.unify ( p.left, p.right );
	}

	fireTableRowsDeleted ( p_row, p_row );
    }


    /** The attribute (vector) "entries" contains objects of this class */
    private static class PairOfTerm {
	public PairOfTerm () {}

	public PairOfTerm ( Term p_left, Term p_right ) {
	    left  = p_left;
	    right = p_right;
	}
	
	public Term left;
	public Term right;
    }

}
