// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.proof;

import java.io.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.init.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.gui.Main;
import de.uka.ilkd.key.gui.PresentationFeatures;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.util.Debug;
import javax.swing.JOptionPane;
import de.uka.ilkd.key.rule.Taclet;

public class ProblemLoader implements Runnable {

   File file;
   Main main;
   
   Proof proof = null;
   IteratorOfNode children = null;
   Node currNode = null;
   Goal currGoal = null;
   Taclet currTaclet = null;
   int currFormula = 0;
   PosInTerm currPosInTerm = PosInTerm.TOP_LEVEL;
   
   java.util.Stack stack = new java.util.Stack();
   
   public ProblemLoader(File file, Main main) {
      this.main = main;
      this.file = file;
   }

   public void run() {
      loadProblem();
   }


   /** parses the given file
    */
   protected void loadProblem() {
      try{
          main.mediator().stopInterface(false); // just the cursor
	  ProblemInitializer init = 
             new ProblemInitializer(new KeYFile("User defined problem", file, main), 
				    main.proofSettings(), main);
	  init.initializeStandardRules();
          main.setStatusLine("Loading problem");
	  init.setUpProof();
	  proof=main.mediator().getSelectedProof();


//************************** THE PROOF ******************************


          main.mediator().stopInterface(true); // now stop the listeners
          main.setStatusLine("Loading proof");
          
          currNode = proof.root(); // initialize loader
          children = currNode.childrenIterator(); // --"--
          init.tryReadProof(this);

          main.mediator().startInterface(true);
          main.setStandardStatusLine();

      } catch (antlr.ANTLRException e) {
          main.mediator().startInterface(true);
          main.setStatusLine("Failed to load problem/proof");
	  e.printStackTrace();
	  JOptionPane.showMessageDialog(main,"Could not load problem/proof " +
            " due to a parser error. The file may be corrupted." +
	    "\nCause: \n" + e.getMessage(), "Parser Error",	
					JOptionPane.ERROR_MESSAGE);    	
      } catch (Exception e) {
	  e.printStackTrace();
          main.setStatusLine("Failed to load problem/proof");
          main.mediator().startInterface(true);
	  JOptionPane.showMessageDialog(main, "Could not load problem/proof." 
					+ "\nCause: \n" 
					+ e.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);   
      }
      finally {
          main.mediator().startInterface(true);
      }
   }


   public void beginExpr(char id, String s) {
//System.out.println("start "+id+"="+s);
      switch (id) {
         case 'b' : stack.push(children);
                    if (children.hasNext()) currNode = children.next();
                    break;

         case 'r' : if (currNode == null) currNode = children.next();
                    // otherwise we already fetched the node at branch point
                    currGoal      = proof.getGoal(currNode);
                    Debug.out("Selected goal ", currGoal);
                    currTaclet    = currGoal.
                                    indexOfTaclets().
                                    lookup(s).
                                    taclet();
                    Debug.out("Applying taclet ", currTaclet.name());
                    currFormula   = 0;
                    currPosInTerm = PosInTerm.TOP_LEVEL; // default for next
                    break;

         case 'f' : currFormula   = Integer.parseInt(s);
                    break;

         case 't' : currPosInTerm = PosInTerm.parseReverseString(s);
                    break;
      }
   }


   public void endExpr(char id) {
//System.out.println("end "+id);
      switch (id) {
         case 'b' : children = (IteratorOfNode) stack.pop();
                    break;
         case 'r' : PosInOccurrence pos = null;
                    if (currFormula != 0) {
                       pos = PosInOccurrence.findInSequent(currGoal.seq(),
                                                           currFormula,
                                                           currPosInTerm);
                    }
                    main.mediator().selectedTaclet(currTaclet, currGoal, pos);
                    children = currNode.childrenIterator();
                    currNode = null;
                    break;
         case 'f' : ;
                    break;
         case 't' : ;
                    break;
      }
   }

}
