// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/**
 * container for the application part of an Taclet. It contains an
 * if-sequence, a list of new variables and a list of variable pairs
 * inidcating the NotFreeIn relation and a list of program variables to
 * be added to the program context.
 */

package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.Sequent;
public class TacletApplPart {

    private Sequent ifseq;
    private ListOfNewVarcond varsNew;
    private ListOfNotFreeIn varsNotFreeIn;

    /** constructs a new TacletApplPart object with the given Sequent, a
     * list of variables that are new and a list of variable pairs
     * that represent the NotFreeIn relation.
     */
    public TacletApplPart(Sequent ifseq, ListOfNewVarcond varsNew,
			ListOfNotFreeIn varsNotFreeIn) {
	this.ifseq=ifseq;
	this.varsNew=varsNew;
	this.varsNotFreeIn=varsNotFreeIn;
    }

    /** returns the if-sequence of the application part of an Taclet */
    public Sequent ifseq() {
	return ifseq;
    } 

    /** returns the list of variables that are new in an Taclet */
    public ListOfNewVarcond varsNew() {
	return varsNew;
    } 

    /** returns the list of variable pairs that represent the
     * NotFreeIn relation of an Taclet
     */
    public ListOfNotFreeIn varsNotFreeIn() {
	return varsNotFreeIn;
    } 


}
