// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule;

import junit.framework.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.logic.op.*;
import java.io.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.rule.inst.*;


/** class tests the apply methods in Taclet
*/
public class TestApplyTaclet extends TestCase{

    String[] strs={"","(A -> B) -> (!(!(A -> B)))",
		   "","all z:s.p(z)",
		   "(A -> B) -> (!(!(A -> B)))","(A -> B) -> (!(!(A -> B)))",
		   "(A -> B) -> (!(!(A -> B)))","",
		   "","<{try{while (1==1) {if (1==2) {break;}} return 1==3; int i=17; } catch (Exception e) { return null;}}>A"};
    Proof[] proof=new Proof[strs.length/2];

    public TestApplyTaclet(String name) {
	super(name);
    }

    private static Semisequent parseTermForSemisequent(String t) {
	if (t.equals("")) { 
	    return Semisequent.EMPTY_SEMISEQUENT;
	}
	ConstrainedFormula cf0
	    = new ConstrainedFormula(TacletForTests.parseTerm(t));
	return Semisequent.EMPTY_SEMISEQUENT.insert(0, cf0);
    }

    public void setUp() {
	TacletForTests.setStandardFile("proofExamples"
				       +File.separator+"testrules.key");
	TacletForTests.parse();
	for (int i=0; i<proof.length; i++) {
	    Semisequent antec = parseTermForSemisequent(strs[2*i]);
	    Semisequent succ = parseTermForSemisequent(strs[2*i+1]);
	    Sequent s = Sequent.createSequent(antec, succ);
	    proof[i]=new Proof();
	    proof[i].setRoot(new Node(proof[i], s));
	}
	
    }
    
    public void testSuccTacletWithoutIf() {
	Term fma=proof[0].root().sequent().succedent().getFirst().formula();
	NoPosTacletApp impright=TacletForTests.getRules().lookup("imp_right");
	TacletAppIndex index=new TacletAppIndex(new TacletIndex());
	index.addNoPosTacletApp(impright);
	Goal goal=new Goal(proof[0].root(), new RuleAppIndex(index));
	PosInOccurrence applyPos= new 
			PosInOccurrence(goal.seq().succedent().getFirst(), 
					PosInTerm.TOP_LEVEL,
					goal.seq());
	ListOfTacletApp rApplist=goal.ruleAppIndex().
		    getTacletAppAt(SLListOfHeuristic.EMPTY_LIST, true, applyPos, null);	
	assert("Too many or zero rule applications.",rApplist.size()==1);
	RuleApp rApp=rApplist.head();
	assert("Rule App should be complete", rApp.complete());
	ListOfGoal goals=rApp.execute(goal, TacletForTests.javaInfo());
	assert("Too many or zero goals for imp-right.",goals.size()==1);	
	Sequent seq=goals.head().node().sequent();
	assertEquals("Wrong antecedent after imp-right",
		     seq.antecedent().getFirst().formula(), fma.sub(0));  
       	assertEquals("Wrong succedent after imp-right",
		     seq.succedent().getFirst().formula(), fma.sub(1));  	
    }
    

    //Vladimir: Please remove the `x as soon as it is fixed.
    public void xtestAddingRule() {
	Term fma=proof[0].root().sequent().succedent().getFirst().formula();
	NoPosTacletApp imprightadd 
	    = TacletForTests.getRules().lookup("TestApplyTaclet_imp_right_add");
	TacletAppIndex index=new TacletAppIndex(new TacletIndex());
	index.addNoPosTacletApp(imprightadd);
	Goal goal=new Goal(proof[0].root(), new RuleAppIndex(index));
	PosInOccurrence applyPos= new 
			PosInOccurrence(goal.seq().succedent().getFirst(), 
					PosInTerm.TOP_LEVEL,
					goal.seq());
	ListOfTacletApp rApplist=goal.ruleAppIndex().
		    getTacletAppAt(SLListOfHeuristic.EMPTY_LIST, true, applyPos, null);	
	assert("Too many or zero rule applications.",rApplist.size()==1);
	RuleApp rApp=rApplist.head();
	assert("Rule App should be complete", rApp.complete());
	ListOfGoal goals=rApp.execute(goal, TacletForTests.javaInfo());
	assert("Too many or zero goals for imp_right_add.",goals.size()==1);
	Sequent seq=goals.head().node().sequent();
	assertEquals("Wrong antecedent after imp_right_add",
		     seq.antecedent().getFirst().formula(),
		     fma.sub(0));  
       	assertEquals("Wrong succedent after imp_right_add",
		     seq.succedent().getFirst().formula(),
		     fma.sub(1));  
	ListOfNoPosTacletApp nfapp=goals.head().indexOfTaclets().getNoFindTaclet
	    (true, SLListOfHeuristic.EMPTY_LIST, null);
	Term aimpb=TacletForTests.parseTerm("A -> B");
	assert("Cut Rule should be inserted to TacletIndex.", nfapp.size()==1);
	assert("Inserted cut rule's b should be instantiated to A -> B.",
	       nfapp.head().instantiations().getInstantiation
	     ((SchemaVariable)TacletForTests.getVariables().lookup(new Name("b")))
	       .equals(aimpb));
	assert("Rule App should be complete", rApp.complete());
	goals=nfapp.head().execute(goals.head(), TacletForTests.javaInfo());
	Sequent seq1=goals.head().seq();
	Sequent seq2=goals.tail().head().seq();
	assert("Preinstantiated cut-rule should be executed", goals.size()==2);
	assert("A->B should be in the succedent of one of the new goals now, "
	       +"it's in the antecedent, anyway.",
	       seq1.succedent().getFirst().formula().equals(aimpb) || 
	       seq2.succedent().getFirst().formula().equals(aimpb) ||
	       (seq1.succedent().get(1)!=null 
		&& seq1.succedent().get(1).formula().equals(aimpb)) ||
	       (seq2.succedent().get(1)!=null 
		&& seq2.succedent().get(1).formula().equals(aimpb)));
    }

    
    public void testSuccTacletAllRight() {
	NoPosTacletApp allright = TacletForTests.getRules().lookup("all_right");
	TacletAppIndex index=new TacletAppIndex(new TacletIndex());
	index.addNoPosTacletApp(allright);
	Goal goal=new Goal(proof[1].root(), new RuleAppIndex(index));
	PosInOccurrence applyPos
	    = new PosInOccurrence(goal.seq().succedent().getFirst(), 
				  PosInTerm.TOP_LEVEL,
				  goal.seq());	
	ListOfTacletApp rApplist=goal.ruleAppIndex().
		    getTacletAppAt(SLListOfHeuristic.EMPTY_LIST, true, applyPos, null);
	assert("Too many or zero rule applications.", rApplist.size()==1);
	RuleApp rApp=rApplist.head();
	assert("Rule App should be complete", rApp.complete());
	ListOfGoal goals=rApp.execute(goal, TacletForTests.javaInfo());
	assert("Too many or zero goals for all-right.",goals.size()==1);	
	Sequent seq=goals.head().seq();
	assertEquals("Wrong antecedent after all-right",seq.antecedent(),
		     Semisequent.EMPTY_SEMISEQUENT);  
       	assertEquals("Wrong succedent after all-right (op mismatch)",
		     seq.succedent().getFirst().formula().op(),
		     TacletForTests.getFunctions().lookup(new Name("p")));  	
    }
    
    public void testTacletWithIf() {
	NoPosTacletApp close = TacletForTests.getRules().lookup("close_goal");
	TacletAppIndex index = new TacletAppIndex(new TacletIndex());
	index.addNoPosTacletApp(close);	
	Goal goal=new Goal(proof[2].root(), new RuleAppIndex(index));
	PosInOccurrence applyPos=new PosInOccurrence
	    (goal.seq().succedent().getFirst(), 
	     PosInTerm.TOP_LEVEL,
	     goal.seq());
 	ListOfTacletApp rApplist
	    = goal.ruleAppIndex().getTacletAppAt(SLListOfHeuristic.EMPTY_LIST, 
					       true, applyPos, null);
 	assert("Too many or zero rule applications.\napp list:"+rApplist,rApplist.size()==1);

 	TacletApp rApp=rApplist.head();
 	ListOfSVInstantiations svilist=rApp.taclet().matchIf(goal.seq(),
					   rApp.instantiations(), null);
	assert("Match Failed.", svilist!=SLListOfSVInstantiations.EMPTY_LIST);
	assert("Too many matches.", svilist.size()==1);
	assert("Wrong match found.", svilist.head()==rApp.instantiations());
	assert("Rule App should be complete", rApp.complete());
 	ListOfGoal goals=rApp.execute(goal, TacletForTests.javaInfo());
	assert("Too many goals for close.", goals.size()==0);		
    }

    
    public void testAntecTacletWithoutIf() {
	Term fma=proof[3].root().sequent().antecedent().getFirst().formula();
	NoPosTacletApp impleft = TacletForTests.getRules().lookup("imp_left");
 	TacletAppIndex index=new TacletAppIndex(new TacletIndex());
 	index.addNoPosTacletApp(impleft);
 	Goal goal=new Goal(proof[3].root(), new RuleAppIndex(index));
	PosInOccurrence applyPos= new PosInOccurrence
	    (goal.seq().antecedent().getFirst(), 
	     PosInTerm.TOP_LEVEL,
	     goal.seq());
 	ListOfTacletApp rApplist
	    = goal.ruleAppIndex().getTacletAppAt(SLListOfHeuristic.EMPTY_LIST, 
					       true, applyPos, null);
 	assert("Too many or zero rule applications.",rApplist.size()==1);
 	RuleApp rApp=rApplist.head();
	assert("Rule App should be complete", rApp.complete());
 	ListOfGoal goals=rApp.execute(goal, TacletForTests.javaInfo());
 	assert("Too many or zero goals for imp-left.",goals.size()==2);	
	Sequent seq=goals.head().node().sequent();
	if (seq.succedent()!=Semisequent.EMPTY_SEMISEQUENT) {
	    assertEquals("Wrong succedent after imp-left",
			 seq.succedent().getFirst().formula(),
			 fma.sub(0));  
	    goals=goals.tail();
	    seq=goals.head().node().sequent();
	    assertEquals("Wrong antecedent after imp-left",
			 seq.antecedent().getFirst().formula(),
			 fma.sub(1));  
	} else {
	    assertEquals("Wrong antecedent after imp-left",
			 seq.antecedent().getFirst().formula(),
			 fma.sub(1));  
	    goals=goals.tail();
	    seq=goals.head().node().sequent();

	    assertEquals("Wrong succedent after imp-left",
			 seq.succedent().getFirst().formula(),
			 fma.sub(0));  
	}
    }


    
    public void testRewriteTacletWithoutIf() {
	NoPosTacletApp contradiction = TacletForTests.getRules().lookup
	    ("TestApplyTaclet_contradiction");
	TacletAppIndex index=new TacletAppIndex(new TacletIndex());
 	index.addNoPosTacletApp(contradiction);
	Goal goal=new Goal(proof[0].root(), new RuleAppIndex(index));
	PosInOccurrence pos
	    = new PosInOccurrence(goal.seq().succedent().getFirst(), 
				  PosInTerm.TOP_LEVEL.down(1).down(0).down(0),
				  goal.seq());
 	ListOfTacletApp rApplist=goal.ruleAppIndex().
	    getTacletAppAt(SLListOfHeuristic.EMPTY_LIST, true, pos, null);	

	assert("Too many or zero rule applications.",rApplist.size()==1);
	RuleApp rApp=rApplist.head();
	assert("Rule App should be complete", rApp.complete());
	ListOfGoal goals=rApp.execute(goal, TacletForTests.javaInfo());
	assert("Too many or zero goals for contradiction.",goals.size()==1);	
	Sequent seq=goals.head().seq();
	Term term=seq.succedent().getFirst().formula().sub(1).sub(0).sub(0);
	assert(term.equals(TacletForTests.parseTerm("!B -> !A")));
    }


    public void testNoFindTacletWithoutIf() {
	NoPosTacletApp cut = TacletForTests.getRules().lookup("TestApplyTaclet_cut");
	TacletAppIndex index=new TacletAppIndex(new TacletIndex());
	Term t_c=TacletForTests.parseTerm("C");
 	index.addNoPosTacletApp(cut);
	Goal goal=new Goal(proof[0].root(), new RuleAppIndex(index));
	PosInOccurrence pos
	    = new PosInOccurrence(goal.seq().succedent().getFirst(), 
				  PosInTerm.TOP_LEVEL,
				  goal.seq());
 	ListOfTacletApp rApplist=goal.ruleAppIndex().
 		    getTacletAppAt(SLListOfHeuristic.EMPTY_LIST, true, pos, null);	
	assert("Too many or zero rule applications.",rApplist.size()==1);
	TacletApp rApp=rApplist.head().addInstantiation
	    ((SchemaVariable)TacletForTests.getVariables().lookup(new Name("b")), t_c);
	assert("Rule App should be complete", rApp.complete());
 	ListOfGoal goals=rApp.execute(goal, TacletForTests.javaInfo());
 	assert("Too many or too few goals.",goals.size()==2);	
 	Sequent seq1=goals.head().node().sequent();
 	goals=goals.tail();
 	Sequent seq2=goals.head().node().sequent();
	if (seq1.antecedent()!=Semisequent.EMPTY_SEMISEQUENT 
	    && seq1.antecedent().getFirst().formula().equals(t_c)) {
	    assert("C is in antecedent of 1st goal but not in succedent of 2nd",
		   seq2.succedent().getFirst().formula().equals(t_c)
		   || seq2.succedent().get(1).formula().equals(t_c));
	} else {
	    assert("C is not in antecedent and not in succedent "
		   +"of first new goal",
		   seq1.succedent().getFirst().formula().equals(t_c)
		   || seq1.succedent().get(1).formula().equals(t_c));	    
	    assert("C is in succedent of first new goal, but not in antecedent "
		   +"of second new goal",
		   seq2.antecedent().getFirst().formula().equals(t_c));
	}
    }




    /*    
    public String automaticProof(Sequent initSeq, TacletIndex index){
	String out="";
	Proof proof=new Proof();
	proof.setRoot(new Node(proof, initSeq));
	ListOfGoal goals=SLListOfGoal.EMPTY_LIST;
	Goal goal=new Goal(proof.root(),new RuleAppIndex(index));
	goals=goals.prepend(goal);
	while (goals.size()!=0) {
	    ConstrainedFormula cfma=null;
	    ConstrainedFormula userCfma=null;   // in the real system the 
		                              //user would select this
	    ListOfTacletApp rapplist=SLListOfTacletApp.EMPTY_LIST;
	    out="\n"+out+("Goals: "+goals+"\n");
	    goal=goals.head();
	    IteratorOfConstrainedFormula
		it=goal.node().sequent().antecedent().iterator();
	    while (it.hasNext()) {
		userCfma=it.next();
		rapplist=rapplist.prepend(goal.ruleAppIndex().
		    getTacletAppAtAndBelow(SLListOfHeuristic.EMPTY_LIST, true, new 
			PosInOccurrence(userCfma, PosInTerm.TOP_LEVEL,
					goal.node().sequent())));
	    }
       	    if (rapplist==SLListOfTacletApp.EMPTY_LIST) {
		it=goal.node().sequent().succedent().iterator();
		while (it.hasNext()) {		
		    userCfma=it.next();	
		    rapplist=rapplist.prepend(goal.ruleAppIndex()
			.getTacletAppAtAndBelow(SLListOfHeuristic.EMPTY_LIST, true, new 
			    PosInOccurrence(userCfma, PosInTerm.TOP_LEVEL,
					    goal.node().sequent()))) ;
		}
	    }
	    out="\n"+out+("GOAL INDEX:"+goal.indexOfTaclets());
	    out="\n"+out+("Rule apps: "+rapplist);
	    out="\n"+out+("Choose for if-test: "+rapplist.head()+"\n");
	    goals=goals.tail();
	    boolean executed=false;
	    while (!executed && rapplist!=SLListOfTacletApp.EMPTY_LIST) {
		ListOfMapFromSchemaVariableToTerm
		    mrlist=((Taclet)(rapplist.head().rule())).matchIf(goal.node().sequent(), 
								    rapplist.head().instantiations());
		out="\n"+out+("List of if-seq matches:"+mrlist);
		if (mrlist!=SLListOfMapFromSchemaVariableToTerm.EMPTY_LIST) {		
		    out+="Execute: "+rapplist.head()+"\n";
		    goals=goals.prepend(rapplist.head().execute(goal));	
		    executed=true;
		}
		rapplist=rapplist.tail();
	    }
	    out="\n"+out+("Tree: "+proof.root()+"\n *** \n");
	    if (!executed) {
		return out+"\nPROOF FAILED.";
	    }
	}
	if (goals.size()==0) out=out+"\nPROOF.";
	return out;
    }


    public void testNatAutomatically() {
	TacletAppIndex index=new TacletAppIndex(new TacletIndex());
      	index.addTaclet(TacletForTests.getRules().lookup("close_goal"));
	index.addTaclet(TacletForTests.getRules().lookup("imp_left"));
	index.addTaclet(TacletForTests.getRules().lookup("imp_right"));
	index.addTaclet(TacletForTests.getRules().lookup("not_left"));
	index.addTaclet(TacletForTests.getRules().lookup("not_right"));
	index.addTaclet(TacletForTests.getRules().lookup
		      ("TestApplyTaclet_predsuccelim"));
	index.addTaclet(pluszeroelim);
	index.addTaclet(zeropluselim);
	index.addTaclet(succelim);
	index.addTaclet(switchsecondsucc);
	index.addTaclet(switchfirstsucc);
	index.addTaclet(closewitheq);
	String s=(automaticProof(seq_testNat, index));
	assert("Automatic proof with nats failed",
	       s.substring(s.length()-6).equals("PROOF."));
    }

*/

    
    public void testIncompleteNoFindTacletApp() {
	NoPosTacletApp cut = TacletForTests.getRules().lookup("TestApplyTaclet_cut");
	assert("TacletApp should not be complete, as b is not instantiated",
	       !cut.complete());
	SchemaVariable b=(SchemaVariable)
	    TacletForTests.getVariables().lookup(new Name("b"));
	assert("b should be in the set of not instantiated SVs",
	       cut.uninstantiatedVars().contains(b));
    }

    public void testIncompleteSuccTacletApp() {
	TacletApp orright = TacletForTests.getRules().lookup("or_right");
	assert("TacletApp should not be complete, as SVs are not instantiated",
	       !orright.complete());

	SchemaVariable b=(SchemaVariable)
	    TacletForTests.getVariables().lookup(new Name("b"));
	SchemaVariable c=(SchemaVariable)
	    TacletForTests.getVariables().lookup(new Name("c"));
	assert("b and c should be in the set of not instantiated SVs",
	       orright.uninstantiatedVars()
	       .equals(SetAsListOfSchemaVariable.EMPTY_SET.add(b).add(c)));
	orright=orright.addInstantiation(b,TacletForTests.parseTerm("A"));
	assert("TacletApp should not be complete, as B is not instantiated",
	       !orright.complete());
	orright=orright.addInstantiation(c,TacletForTests.parseTerm("B"));
	assert("TacletApp should not be complete, as Position unknown",
	       !orright.complete());
	Sequent seq=proof[0].root().sequent();
	orright=orright.setPosInOccurrence
	    (new PosInOccurrence(seq.succedent().get(0),
				 PosInTerm.TOP_LEVEL,
				 seq));
	assert("TacletApp should now be complete with Position set and SVs "
	       +"instantiated",
	       orright.complete());


    }

    
    public void testPrgTacletApp() {
	NoPosTacletApp wh0 = TacletForTests.getRules().lookup("TestApplyTaclet_while0");
	SchemaVariable e2=(SchemaVariable)
	    TacletForTests.getVariables().lookup(new Name("#e2"));
	SchemaVariable p1=(SchemaVariable)
	    TacletForTests.getVariables().lookup(new Name("#p1"));
	//	wh0=wh0.addInstantiation(e2,TacletForTests.parseExpr("boolean", "false"));
	//	wh0=wh0.addInstantiation(p1,TacletForTests.parsePrg("{if (false){}}"));
	Sequent seq=proof[4].root().sequent();
	PosInOccurrence pio=new PosInOccurrence(seq.succedent().get(0),
						       PosInTerm.TOP_LEVEL,
						seq);
	TacletAppIndex index=new TacletAppIndex(new TacletIndex());
	index.addNoPosTacletApp(wh0);
	Goal goal=new Goal(proof[4].root(), new RuleAppIndex(index));
	ListOfTacletApp rApplist=goal.ruleAppIndex().
	    getTacletAppAt(SLListOfHeuristic.EMPTY_LIST, true, pio, null);
	ListOfGoal goals=rApplist.head().execute(goal, TacletForTests.javaInfo());
    }

    
}
