// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.statement.MethodBodyStatement;
import de.uka.ilkd.key.java.reference.MethodReference;
import de.uka.ilkd.key.java.declaration.Modifier;
import de.uka.ilkd.key.java.declaration.MethodDeclaration;
import de.uka.ilkd.key.java.declaration.LocalVariableDeclaration;
import de.uka.ilkd.key.java.declaration.VariableSpecification;
import de.uka.ilkd.key.java.declaration.ParameterDeclaration;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.ArrayOfStatement;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.op.ProgramMethod;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.java.visitor.ProgVarReplaceVisitor;

public class ExpandMethodBody extends ProgramMetaConstruct {

    public ExpandMethodBody(SchemaVariable sv) {
	super(new Name("expand-method-body"), sv);
    }


    /** 
     * Replaces the MethodBodyStatement shortcut with the full body,
     * performs prefix adjustments in the body (execution context).
     * @param javaInfo the JavaInfo with all necessary information 
     * about the java programs
     * @param svInst the instantiations esp. of the inner and outer label 
     * @return the transformed program
     */
    public ProgramElement symbolicExecution(ProgramElement pe,
					    JavaInfo javaInfo,
					    SVInstantiations svInst) {

	MethodBodyStatement mbs = (MethodBodyStatement) pe;
        MethodReference mr = mbs.getMethodReference();
        ProgramMethod pm = mr.method(javaInfo);
	MethodDeclaration methDecl = pm.getMethodDeclaration();
	int params = methDecl.getParameterDeclarationCount();
        Statement[] paramDecl = new Statement[params];
	for (int i = 0; i < params; i++) {
	    ParameterDeclaration parDecl = 
		methDecl.getParameterDeclarationAt(i);
	    VariableSpecification originalSpec = 
		parDecl.getVariableSpecification();
	    VariableSpecification newSpec = 
		new VariableSpecification
		    (originalSpec.getProgramVariable(), 
		     originalSpec.getDimensions(), 
                     mr.getArgumentAt(i),
                     originalSpec.getType());
	    paramDecl[i] = new LocalVariableDeclaration
		(parDecl.getModifiers(), parDecl.getTypeReference(), newSpec);
	    
	} 

	StatementBlock bodyWithParams = 
           insertStatementInBlock(paramDecl, 
				  (StatementBlock) mbs.getBody(javaInfo));

	SetExecutionContextVisitor repl = 
           new SetExecutionContextVisitor(bodyWithParams,
                                          mbs.getDesignatedContext(),
                                          mbs.isStatic(javaInfo));
        repl.start();
        Statement result = (Statement) repl.result(); 
	ProgVarReplaceVisitor paramRepl =
	    new ProgVarReplaceVisitor(result);
	paramRepl.start();
	return (Statement) paramRepl.result();
        
    }



    /** inserts the given statements at the begin of the block 
     * @param stmnt array of Statement those have to be inserted
     * @param b the Statementblock where to insert
     */
    private StatementBlock insertStatementInBlock(Statement[] stmnt, 
						  StatementBlock b) {
	
	Statement[] block = new Statement[b.getStatementCount()+
					  stmnt.length];
	for (int j = 0; j < stmnt.length; j++) {
	    block[j] = stmnt[j];
	}
	for (int i = 0; i < b.getStatementCount(); i++) {
	    block[i+stmnt.length] = b.getStatementAt(i);
	}
	return new StatementBlock(new ArrayOfStatement(block));	
    }
}
