// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.rule.metaconstruct;

import java.util.Stack;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.statement.MethodBodyStatement;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.java.visitor.CreatingASTVisitor;

/**
 * Walks through a java AST in depth-left-first-order. This visitor
 * replaces ThisReferences by a variable this.
 */
public class SetExecutionContextVisitor extends CreatingASTVisitor {

    private ProgramElement result=null;

    private ReferencePrefix newPrefix = null;
    
    private boolean staticCall;

    /** create the  ThisReplaceVisitor
     * @param root the ProgramElement where to begin
     */
    public SetExecutionContextVisitor(Statement st, 
                                      ReferencePrefix newPrefix,
                                      boolean staticCall) {
	super(st);
	this.newPrefix  = newPrefix;
        this.staticCall = staticCall;
    }

    /** the action that is performed just before leaving the node the
     * last time 
     */
    protected void doAction(ProgramElement node) {
	node.visit(this);
    }
    
    /** starts the walker*/
    public void start() {	
	stack.push(new ExtList());		
	walk(root());
	ExtList el=(ExtList)stack.peek();
	int i=0;
	while (!(el.get(i) instanceof ProgramElement)) {
	    i++;
	}
	result=(ProgramElement) ((ExtList)stack.peek()).get(i);
    }

    public ProgramElement result() { 	
	return result;
    }
    
    /** walks through the AST. While keeping track of the current node
     * @param node the JavaProgramElement the walker is at 
     */
    protected void walk(ProgramElement node) {	
	stack.push(new ExtList());
	super.walk(node);	
    }


// ************ HERE'S THE MEAT *********************************


   private void adjustPrefix(ReferencePrefix r) {
      ReferencePrefix nextChunk;
      do {
         nextChunk = r.getReferencePrefix();
         if (nextChunk instanceof ThisReference) {// this is always replaced
            r.setReferencePrefix(newPrefix);
            break; // once replaced -- stop processing
         }
         else 

// this was too aggressive; breaks correspondence with formulae

//         if (nextChunk instanceof ProgramVariable) { // should only appear @0
//            if (staticCall) r.setReferencePrefix(newPrefix);
//            break;
//         }
//         else 
//         if (nextChunk instanceof FieldReference) { // replace on static
//            if ((nextChunk.getReferencePrefix() == null) && staticCall) {
//               r.setReferencePrefix(newPrefix);
//               break;
//            }
//         }
//         else 
         if (nextChunk == null) { // add prefix here
            if (r instanceof MethodReference) r.setReferencePrefix(newPrefix);
            else 
            if (r instanceof FieldReference)  r.setReferencePrefix(newPrefix);
            else 
            System.err.println("INFO: prefix root is: "+
                               r.getClass());
            // this is the end of the iteration so no break necessary
         }
         // and finally...
         if (nextChunk != null) r = nextChunk;
      } while (nextChunk != null);
   }



    public void performActionOnThisReference(ThisReference x) {
	addChild(newPrefix);
	changed();
    }



    public void performActionOnMethodReference(MethodReference mr) {
	Debug.out("mr-vorher: ",stack);
       adjustPrefix(mr);
       doDefaultAction(mr);
       Debug.out("mr-nachher: ",stack);
    }




    public void performActionOnFieldReference(FieldReference fr) {
	Debug.out("fr-vorher: ",stack);
       adjustPrefix(fr);
       doDefaultAction(fr);
       Debug.out("fr-nachher: ",stack);
    }


}
