// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.metaconstruct;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.java.ArrayOfStatement;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.declaration.ParameterDeclaration;
import de.uka.ilkd.key.java.expression.operator.Instanceof;
import de.uka.ilkd.key.java.expression.Assignment;
import de.uka.ilkd.key.java.expression.operator.CopyAssignment;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

/** This class is used to perform program transformations needed 
 * for the symbolic execution of a try-return as 
 * try {return x; alpha} beta; where beta is the catch-finally part
 */
public class TryReturnFinally extends ProgramMetaConstruct {

     /** creates a try-return-to-if ProgramMetaConstruct 
     * @param tryReturn the Statement contained by 
     * the meta construct 
     */
    public TryReturnFinally(Statement tryReturn) {
	super("try-return-finally", tryReturn); 
    }

    /** inserts a statment in the given block
     * @param stmnt the Statement to insert
     * @param b theStatementblock where to insert
     * @param pos int giving the position where to insert
     */
    private StatementBlock insertStatementInBlock(Statement stmnt, 
						  StatementBlock b, 
						  int pos) {
	
	Statement[] block = new Statement[b.getStatementCount()+1];
	int offset = 0;
	for (int i = 0; i < block.length-1; i++) {
	    if (i == pos) {
		block[i] = stmnt;
		offset = 1;
	    } 
	    block[i+offset] = b.getStatementAt(i);
	}
	// may be it should have been insert at last one
	if (offset == 0) {
	    block[pos] = stmnt;
	}
	return new StatementBlock(new ArrayOfStatement(block));	
    }

    /** performs the program transformation needed for symbolic
     * program transformation 
     * @return the transformed program
     */
    public ProgramElement symbolicExecution(ProgramElement pe, 
					    JavaInfo javaInfo,
					    SVInstantiations insts) {
	Try try_stmnt = (Try) pe;
	JavaStatement insertStmt=(JavaStatement) (try_stmnt.getBody().getChildAt(0)); 
	StatementBlock result;
	Finally finally_stmnt = null;
	if (try_stmnt.getBranchAt(try_stmnt.getBranchCount()-1) instanceof Finally) {
	    finally_stmnt = (Finally) try_stmnt.getBranchAt
		(try_stmnt.getBranchCount()-1);
	    StatementBlock finally_block = (StatementBlock)finally_stmnt.getBody();	    
	    result = insertStatementInBlock(insertStmt, finally_block, 
					    finally_block.getStatementCount());
	} else {
	    result = new StatementBlock(insertStmt);
	}
	return result;
    }
}
